//
//
//
//

package Voyager.db.servers;

import java.util.Vector;
import java.sql.*;
import org.omg.CORBA.*;
import Voyager.db.*;
import java.util.Date;

public class VOY_Database_impl extends Voyager.db._VOY_DatabaseImplBase
{
  String burl_;
  String dburl_;
  String driver_;

  VOY_Database_impl(String sURL, String sDriver, String baseURL)
  {
    burl_ = baseURL;
    dburl_ = sURL;
    driver_ = sDriver;
  }

  private void getDriver()
  {
    try {
      Class.forName(driver_);
    } catch(Exception e) {
      System.out.println("Can't find database driver class: " + e);
    }
  }

  private synchronized int getNextSessionID()
  {
    getDriver();

     try {
      String query = new String("select s.id from Session s order by s.id desc");
      Connection con = DriverManager.getConnection(dburl_, "voyager", "");
      Statement stmt = con.createStatement();
      ResultSet rs = stmt.executeQuery(query);

      rs.next();

      int id;

      try {
	  // Get the biggest ID 
	  id = rs.getInt("id");

	  // Create the next ID
	  id++;
      } catch( java.lang.NullPointerException ex ) {
	// The db is empty - create the first id
	System.err.print( "Error on reading last session id: " + ex + " " );
	id = 1;
      }

      stmt.close();
      con.close();

      return(id);
    } catch(SQLException ex) {
      System.err.print("SQL Exception: " + ex + " ");
      return(-1);
    }
  }

  private String getDefaultRecordFS()
  {
    getDriver();

    try {
      String query = new String("select p.pvalue from Parameter p where pkey = 'default_record_filesystem'");
      Connection con = DriverManager.getConnection(dburl_,  "voyager", "");
      Statement stmt = con.createStatement();
      ResultSet rs = stmt.executeQuery(query);

      String val = null;
      while(rs.next())
 	val = rs.getString("pvalue").trim( );
      stmt.close();
      con.close();

      return(val);
    } catch(SQLException ex) {
      System.err.print("SQL Exception: " + ex);
      return(null);
    }
  }

  private String getFSIDForFSName(String name)
  {
    getDriver();

    try {
      String query = new String("select f.fs_id from Filesystem f where name = '"+name+"'");
      Connection con = DriverManager.getConnection(dburl_,  "voyager", "");
      Statement stmt = con.createStatement();
      ResultSet rs = stmt.executeQuery(query);

      String val = null;
      while(rs.next())
 	val = rs.getString("fs_id").trim( );
      stmt.close();
      con.close();

      return(val);
    } catch(SQLException ex) {
      System.err.print("SQL Exception: " + ex);
      return(null);
    }
  }

  public SessionDescriptor[] getSessionList(int demo)
  {
    SessionDescriptor[] sd;

    getDriver();

    try {
      Vector results = new Vector();
      String query;
      if(demo > 0)
	query = new String("select s.name, s.id from Session s, DemoSession d where s.id = d.id order by s.id DESC");
      else 
	query = new String("select s.name, s.id from Session s order by s.id DESC");
      
      Connection con = DriverManager.getConnection(dburl_,  "voyager", "");
      Statement stmt = con.createStatement();
      ResultSet rs = stmt.executeQuery(query);

      while (rs.next()) {
	String name = rs.getString("name").trim( );
	int id = rs.getInt("id");
	System.out.println("Session Description: " + id + " " + name);
	results.addElement(new SessionDescriptor(id, name));
      }
      stmt.close();
      con.close();

      sd = new SessionDescriptor[results.size()];
      results.copyInto(sd);

      return(sd);
    } catch(SQLException ex) {
      System.err.print("SQL Exception: " + ex);
      return(null);
    }
  }

  public String getOwner(int session_id)
  {
    getDriver();

    try {
      Vector results = new Vector();
      String query;
      query = new String("select s.owner from Session s where id=" +
			 session_id);
      
      Connection con = DriverManager.getConnection(dburl_,  "voyager", "");
      Statement stmt = con.createStatement();
      ResultSet rs = stmt.executeQuery(query);

      rs.next();
      String owner = rs.getString("owner").trim( );

      stmt.close();
      con.close();

      return(owner);
    } catch(SQLException ex) {
      System.err.print("SQL Exception: " + ex);
      return(null);
    }
  }

  public String getName(int session_id)
  {
    getDriver();

    try {
      Vector results = new Vector();
      String query;
      query = new String("select s.name from Session s where id=" +
			 session_id);
      
      Connection con = DriverManager.getConnection(dburl_,  "voyager", "");
      Statement stmt = con.createStatement();
      ResultSet rs = stmt.executeQuery(query);

      rs.next();
      String name = rs.getString("name").trim( );

      stmt.close();
      con.close();

      return(name);
    } catch(SQLException ex) {
      System.err.print("SQL Exception: " + ex);
      return(null);
    }
  }

  public String getURL(int session_id)
  {
    String url_ = "rtsp://"+burl_+"/"+session_id;

    return(url_);
  }

  public String getDescription(int session_id)
  {
    getDriver();

    try {
      Vector results = new Vector();
      String query;
      query = new String("select s.description from Session s where id = " + 
			 session_id);
      
      Connection con = DriverManager.getConnection(dburl_,  "voyager", "");
      Statement stmt = con.createStatement();
      ResultSet rs = stmt.executeQuery(query);

      rs.next();
      String name = rs.getString("description").trim( );

      stmt.close();
      con.close();

      return(name);
    } catch(SQLException ex) {
      System.err.print("SQL Exception: " + ex);
      return(null);
    }
  }

  public String getSessionPath(int session_id, String host)
  {
    System.out.println("Getting session path: " +session_id+ " host: " + host);

    getDriver();

    try {
      Vector results = new Vector();
      String query;
      query = new String("select sl.path, fm.mount_point from SessionLocation sl, FilesystemMount fm where sl.session_id = " + session_id + " and sl.fs_id = fm.fs_id and fm.host = '" + host+"'");
      
      Connection con = DriverManager.getConnection(dburl_,  "voyager", "");
      Statement stmt = con.createStatement();
      ResultSet rs = stmt.executeQuery(query);

      rs.next();
      String path = rs.getString("path").trim( );
      String mountpoint = rs.getString("mount_point").trim( );

      stmt.close();
      con.close();

      return( mountpoint + "/" + path );
    } catch(SQLException ex) {
      System.err.print("SQL Exception: " + ex);
      return(null);
    }
  }

  public String getStreamPath(int session_id, String host, String stream_id)
  {
    String session_path = getSessionPath(session_id, host);
    return(session_path+"/"+stream_id);
  }

  public int allocateRecordSession(String name, String description,
				   String source_host)
  {
    int session_id = getNextSessionID();

    System.out.println("Got ID: " + session_id);

    if(session_id != -1)
      {
	getDriver();

	try {
	  Connection con = DriverManager.getConnection(dburl_,  "voyager", "");
	  // Statement stmt = con.createStatement();
	  java.util.Date uD = new java.util.Date( );
	  java.sql.Date d = new java.sql.Date( uD.getTime( ) );
	  java.sql.Time t = new java.sql.Time( uD.getTime( ) );
	  System.out.println( "Date " + d + ", Time " + t );
	  String goodName = processBadChars( name );
	  String goodDesc = processBadChars( description );
	  PreparedStatement pStmt = con.prepareStatement( "insert into session values ( ? , ? , ? , ? , ? , ? , ? ) " );
	  pStmt.setString( 1 , goodName );
	  pStmt.setInt( 2 , session_id );
	  pStmt.setString( 3 , goodDesc );
	  pStmt.setTime( 4 , t );
	  pStmt.setDate( 5 , d );
	  pStmt.setString( 6 , source_host );
	  pStmt.setString( 7 , "voyager_public" );
	  

	  //String query = new String("insert into session values ('" + goodName + "', " + session_id + ", '" + goodDesc + "', " + d.getTime() + ", '" + source_host + "', 'voyager_public')");

	  pStmt.executeUpdate();
	  System.out.println( "after executing stmt" );
	  pStmt.close();
	  System.out.println( "after closing stmt" );
	  con.close();
	  System.out.println( "after closing connection" );

	  return(session_id);
	} catch(SQLException ex) {
	  System.err.print("SQL Exception: " + ex);
	  return(-1);
	}
      } else {
	return(-1);
      }
  }

    // Preceed characters that have significance to sql with a \ char
    private String processBadChars( String inString )
    {
	String outString = "";
	int offset;

	if( inString != null )
	{
	    // Handle the ' char
	    int charIdx;
	    for( offset = 0 ; ( charIdx=inString.indexOf( '\'' , offset ) ) >= 0 ; offset = charIdx + 1 )
	    {
		try {
		    String subString =  inString.substring( offset , charIdx );
		    outString += subString + "\\'";
		} catch( StringIndexOutOfBoundsException ex ) 
		{
		    // System.out.println( "processBadChars:   ex" + ", offset " + offset + ", charIdx " + charIdx + ", inString " + inString );

		    outString += "\\'";
		}
		
		// System.out.println( "processBadChars:  " + inString + " turning into " + outString);
	    }

	    if( offset < inString.length( ) )
		outString +=  inString.substring( offset );
	}

	return outString;
    }

  public int allocateRecordSpace(int session_id)
  {
    System.out.println("ARS");

    String name = getName(session_id);
    String owner = getOwner(session_id);
    String spath = new String(owner+"/"+session_id);

    System.out.println("Name: " + name + " Owner: " + owner + " Path: " +spath);

    if(name != null && owner != null && spath != null)
      {
	String fs_name = getDefaultRecordFS();
	System.out.println("FSname: " +fs_name);
	if(fs_name != null)
	  {
	    String fs_id = getFSIDForFSName(fs_name);
	    System.out.println("FSid: " +fs_id);
	    if(fs_id != null)
	      {
		getDriver();

		try {
		Connection con = DriverManager.getConnection(dburl_,  "voyager", "");
		Statement stmt = con.createStatement();
		String query = new String("insert into SessionLocation values (" + 
					  session_id + ", " + fs_id + ", '" + spath + 
					  "')");
		stmt.executeUpdate(query);
		
		stmt.close();
		con.close();

		return(0);
		} catch (SQLException e) {
		  return(-1);
		}
	      } else {
		return(-1);
	      }
	  } else {
	    return(-1);
	  }
      } else {
	return(-1);
      }
  }

  public String[] getStreamNames(int session_id)
  {
    String[] streams;

    getDriver();

    try {
      Vector results = new Vector();
      String query;
      query = new String("select distinct ssrc from SessionMetadata where id = " + 
			 session_id);
      
      Connection con = DriverManager.getConnection(dburl_,  "voyager", "");
      Statement stmt = con.createStatement();
      ResultSet rs = stmt.executeQuery(query);

      while (rs.next()) {
	String s = rs.getString("ssrc").trim( );
	results.addElement(s);
      }
      stmt.close();
      con.close();

      streams = new String[results.size()];
      results.copyInto(streams);

      return(streams);
    } catch(SQLException ex) {
      System.err.print("SQL Exception: " + ex);
      return(null);
    }
  }

  public StreamKeyVal[] getStreamKeyValues(int session_id, String stream)
  {
    StreamKeyVal[] skv = null;

    getDriver();

    try {
      Vector results = new Vector();
      String query;
      query = new String("select sm.mkey, sm.mvalue from SessionMetadata sm where sm.id = " + session_id + " and ssrc = '" + stream + "'");
      Connection con = DriverManager.getConnection(dburl_,  "voyager", "");
      Statement stmt = con.createStatement();
      ResultSet rs = stmt.executeQuery(query);

      String mkey = null;
      String mval = null;
      while (rs.next()) {
	mkey = rs.getString("mkey").trim( );
	mval = rs.getString("mvalue").trim( );
	results.addElement(new StreamKeyVal(mkey, mval));
      }
      stmt.close();
      con.close();

      skv = new StreamKeyVal[results.size()];
      results.copyInto(skv);

      return(skv);
    } catch(SQLException ex) {
      System.err.print("SQL Exception: " + ex);
      return(null);
    }
  }

  public int setStreamKeyValue(int session_id, String stream,
				 String key, String value)
  { 
    getDriver();

    try {
      Connection con = DriverManager.getConnection(dburl_,  "voyager", "");
      Statement stmt = con.createStatement();
      String query = new String("insert into SessionMetadata values (" + 
				session_id + ", '" + stream + "', '" + 
				key + "', '" + value + "')");
      stmt.executeUpdate(query);
      
      stmt.close();
      con.close();
      
      return(0);
    } catch(SQLException ex) {
      System.err.print("SQL Exception: " + ex);
      return(-1);
    }
  }

  public int createTable(String query)
  {
    getDriver();

    try {
      Connection con = DriverManager.getConnection(dburl_,  "voyager", "");
      Statement stmt = con.createStatement();
      stmt.executeUpdate(query);
	
      stmt.close();
      con.close();

      return(0);
    } catch(SQLException ex) {
      System.err.print("SQL Exception: " + ex);
      return(0);
    }
  }

  public void storeMBoneRecord(int id, String name, String description,
			       int start, int stop, String record)
  {
    getDriver();

    try {
      Connection con = DriverManager.getConnection(dburl_,  "voyager", "");
      Statement stmt = con.createStatement();
      String query = new String("insert into MboneSession values (" + id + 
				", '" + name + "', '" + description + 
				"', " + start + ", " + stop + ", '" + 
				record + "'"); 
      stmt.executeUpdate(query);
	
      stmt.close();
      con.close();

    } catch(SQLException ex) {
      System.err.print("SQL Exception: " + ex);
    }
  }

  public void addMboneSessionForHost(int id, String host)
  {
  }

  public String[][] Query(String query)
  {
    return(null);
  }
}



