package Voyager.rtsp;

import java.io.*;
import java.net.*;

public class TCPClient implements Client
{
    public TCPClient()
    {
	session_ = null;
	handler_ = null;
    }
    
    public ConnectResult connect(String url)
    {
	String host;
	int port;

	try {
	    if (!url.substring(0, 7).equals("rtsp://"))
	    {
		System.out.println("bad url");
	    }

	    String body = url.substring(7);
	    System.out.println("body is " + body);

	    int i = body.indexOf(':');
	    if (i < 0)
		throw new Exception();
	    host = body.substring(0, i);
	    body = body.substring(i + 1);

	    i = body.indexOf('/');
	    if (i < 0)
		throw new Exception();

	    String portStr = body.substring(0, i);
	    port = Integer.parseInt(portStr);
	}
	catch (Exception ex)
	{
	    System.out.println("bad url " + url + " " + ex);
	    ConnectResult rc = new ConnectResult();
	    rc.setSuccess(false);
	    return rc;
	}

	return connect(host, port);
    }
    
    public ConnectResult connect(String host, int port)
    {
	ConnectResult rc = new ConnectResult();

	try
	{
	    handler_ = new TCPHandler(host, port);

	    handlerThread_ = new Thread(handler_);
	    handlerThread_.start();
	}
	catch (UnknownHostException unk)
	{
	    System.out.println("Unknown host " + host);
	}
	catch (IOException iox)
	{
	    System.out.println("IO Exception " + iox);
	}
	
	if (handler_ == null)
	{
	    rc.setSuccess(false);
	}

	return rc;
    }

    public void waitForThread()
    {
	try {
	    handlerThread_.join(0);
	}
	catch (InterruptedException e)
	{
	}
	System.out.println("Thread done");
    }	

    public DescribeSessionResult describeSession(String sessionId)
    {
	DescribeSessionResult rc = new DescribeSessionResult();

	Request request = new Request("DESCRIBE", sessionId);

	Response response = handler_.sendRequest(request);

	if (response.code() != 200)
	{
	    rc.setSuccess(false);
	}
	else
	{
	    Session session = Session.parseResponse(response);
	    if (session == null)
	    {
		rc.setSuccess(false);
	    }
	    else
	    {
		session.setId(sessionId);
		rc.setSession(session);
	    }
	}
	
	return rc;
    }

    public DescribeStreamResult describeStream(String sessionId, String streamId)
    {
	DescribeStreamResult rc = new DescribeStreamResult();

	Request request = new Request("DESCRIBE", sessionId + "/" + streamId);
	Response response = handler_.sendRequest(request);
	
	if (response.code() != 200)
	{
	    rc.setSuccess(false);
	}
	else
	{
	    Stream stream = Stream.parseResponse(response);
	    if (stream == null)
	    {
		rc.setSuccess(false);
	    }
	    else
	    {
		stream.setName(streamId);
		rc.setStream(stream);
	    }
	}
	
	return rc;
    }

    public SetupStreamResult setupStream(String sessionId, String stream,
					 String transportInfo, int rtspSession)
    {
	SetupStreamResult rc = new SetupStreamResult();
	Request request = new Request("SETUP", sessionId + "/" + stream);

	if (rtspSession >= 0)
	{
	    request.setHeader("Session", "" + rtspSession);
	}

	request.setHeader("Transport", transportInfo);

	Response response = handler_.sendRequest(request);

	if (response.code() != 200)
	{
	    rc.setSuccess(false);
	}
	else
	{
	    rc.setSuccess(true);

	    String sessionHdr = (String) response.headers().get("Session");
	    rc.setRtspSessionId(Integer.parseInt(sessionHdr));
	    System.out.println("got session hdr " + rc.rtspSessionId());
	}

	return rc;
    }

    public TeardownStreamResult teardownStream( String sessionId , String stream , int rtspSession )
    {
	TeardownStreamResult rc = new TeardownStreamResult( );
	
	Request request = new Request( "TEARDOWN", sessionId + "/" + stream );
	if (rtspSession >= 0)
	{
	    request.setHeader("Session", "" + rtspSession);
	}
	Response response = handler_.sendRequest( request );

	if( response.code( ) != 200 )
	{
	    rc.setSuccess( false );
	    System.out.println( "sent bad teardown " );
	}
	else
	{
	    rc.setSuccess( true );
	    System.out.println( "sent good teardown " );
	}

	return rc;
    }

    public PlayStreamResult playStream(String sessionId, String stream)
    {
	System.out.println("Play " + sessionId + " " + stream);
	return new PlayStreamResult();
    }

    public PlaySessionResult playSession(String sessionId, int rtspSession)
    {
	System.out.println("Play " + sessionId);
	PlaySessionResult rc = new PlaySessionResult();
	
	Request request = new Request("PLAY", sessionId);

	if (rtspSession >= 0)
	{
	    request.setHeader("Session", "" + rtspSession);
	}

	Response response = handler_.sendRequest(request);

	if (response.code() != 200)
	{
	    rc.setSuccess(false);
	}
	else
	{
	    rc.setSuccess(true);
	}

	return rc;
    }

    public RecordSessionResult recordSession(String sessionId, int rtspSession)
    {
	System.out.println("Record " + sessionId);
	RecordSessionResult rc = new RecordSessionResult();
	
	Request request = new Request("RECORD", sessionId);

	if (rtspSession >= 0)
	{
	    request.setHeader("Session", "" + rtspSession);
	}

	Response response = handler_.sendRequest(request);

	if (response.code() != 200)
	{
	    rc.setSuccess(false);
	}
	else
	{
	    rc.setSuccess(true);
	}

	return rc;
    }

    public PauseStreamResult pauseStream(String sessionId, String stream)
    {
	System.out.println("Pause " + sessionId + " " + stream);
	return new PauseStreamResult();
    }

    Session session_;
    TCPHandler handler_;
    Thread handlerThread_;
}
