package Voyager.rtsp.clients;

import java.awt.*;
import java.awt.event.*;

public class AddressSelectorPanel extends Panel
{
    public static final int AUTOMATIC = 1;
    public static final int MANUAL = 2;
    public static final int SDP = 3;
    public static final int FLBONE = 4;
    public static final int ACTINIDES = 5;
    Checkbox automaticCheckbox_, manualCheckbox_, sdpCheckbox_, flCheckbox_,
	actinCheckbox_;
    CheckboxGroup checkboxGroup_;
    TextArea addressArea_;
    Button backButton_;
    Button nextButton_;
    Button sdpChooseButton_;

    RecordUI controller_;

    int selectionType_;
    
    public AddressSelectorPanel(RecordUI ui)
    {
	super();

	controller_ = ui;
	
	GridBagLayout layout = new GridBagLayout();
	setLayout(layout);

	GridBagConstraints c = new GridBagConstraints();

	checkboxGroup_ = new CheckboxGroup();

	c.gridwidth = GridBagConstraints.REMAINDER;
	c.weightx = 1.0;
	c.fill = GridBagConstraints.HORIZONTAL;
	Label l = new Label("Choose the network address to be used for this recording.");
	layout.setConstraints(l, c);
	add(l);

	//

	Checkbox box;

	flCheckbox_ = box = new Checkbox("FLBONE", checkboxGroup_, false);
	layout.setConstraints(box, c);
	add(box);
	box.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		checkboxStateChanged(e); }});

	actinCheckbox_ = box = new Checkbox("Actinides", checkboxGroup_, false);
	layout.setConstraints(box, c);
	add(box);
	box.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		checkboxStateChanged(e); }});

	manualCheckbox_ = box = new Checkbox("Manual", checkboxGroup_, false);
	layout.setConstraints(box, c);
	add(box);
	box.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		checkboxStateChanged(e); }});

	c.insets.left = 20;
	addressArea_ = new TextArea(3, 30);
	layout.setConstraints(addressArea_, c);
	add(addressArea_);

	c.insets.left = 0;

	automaticCheckbox_ = box = new Checkbox("Automatic", checkboxGroup_, true);
	layout.setConstraints(box, c);
	add(box);
	// We don't support this yet.
	box.setEnabled(false);
	box.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		checkboxStateChanged(e); }});

	sdpCheckbox_ = box = new Checkbox("SDP", checkboxGroup_, false);
	layout.setConstraints(box, c);
	add(box);
	// We don't support this yet.
	box.setEnabled(false);
	box.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		checkboxStateChanged(e); }});

	c.insets.left = 20;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.WEST;
	Button button = new Button("Choose...");
	sdpChooseButton_ = button;
	layout.setConstraints(button, c);
	add(button);
	button.setEnabled(false);

	c.insets.left = 4;
	c.anchor = GridBagConstraints.SOUTHEAST;

	c.weightx = 1.0;
	c.weighty = 1.0;
	c.gridwidth = 1;

	backButton_ = new Button("< Back");
	layout.setConstraints(backButton_, c);
	add(backButton_);

	c.weightx = 0.0;
	c.weighty = 1.0;
	c.gridwidth = GridBagConstraints.RELATIVE;

	nextButton_ = new Button("Create Session");
	layout.setConstraints(nextButton_, c);
	add(nextButton_);

	// Register the UI

	ActionListener listener = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		controller_.addressPanelDone();
	    }
	};

	nextButton_.addActionListener(listener);

	listener = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		controller_.addressPanelBack();
	    }
	};

	backButton_.addActionListener(listener);


	c.weightx = 0.0;
	c.gridwidth = GridBagConstraints.REMAINDER;

	button = new Button("Help");
	layout.setConstraints(button, c);
	add(button);
	button.setEnabled(false);

	checkboxGroup_.setSelectedCheckbox( flCheckbox_ );
	
	// Set enable state
	checkboxStateChanged(null);
    }

    Button nextButton() { return nextButton_; }

    //
    // We can't just bind the event to a different method for
    // each checkbox since we can't reliably tell the state
    // transitions when the checkboxes are part of a group.
    //
    void checkboxStateChanged(ItemEvent e)
    {
	System.out.println( "In checkboxStateChanged" );
	Checkbox selected;

	selected = checkboxGroup_.getSelectedCheckbox();
	
	sdpChooseButton_.setEnabled(false);
	automaticCheckbox_.setEnabled( false );
	addressArea_.setEnabled( false );

	if (selected == automaticCheckbox_)
	{
	    selectionType_ = AUTOMATIC;
	}
	else if (selected == flCheckbox_)
	{
	    selectionType_ = FLBONE;
	}
	else if (selected == actinCheckbox_)
	{
	    selectionType_ = ACTINIDES;
	}
	else if (selected == manualCheckbox_)
	{
	    addressArea_.setEnabled(true);
	    selectionType_ = MANUAL;
	}
	if (selected == sdpCheckbox_)
	{
	    selectionType_ = SDP;
	}
	System.out.println( "Selection: " + selectionType_ );
    }

    public int getSelectionType() { return selectionType_; }

    public String getManualAddress() { return addressArea_.getText(); }
}

