package Voyager.rtsp.clients;

import java.awt.*;
import java.awt.event.*;
import java.util.*;import java.util.Vector;
import Voyager.rtsp.*;
import Voyager.util.*;
import java.text.*;
import Voyager.db.clients.*;
import java.net.*;


public class PlaybackManager extends Panel implements ActionListener, Closable {

  RTPFormatTable RTPFormats_;
  RtspURL url_;

  // Define subcontainers
  Panel         sessionDescPanel , transcodePanel , vcrPanel, streamListPanel;
  ScrollPane    streamListPane;

  // VCR controls 
  Button        playButton , stopButton , quitSessionButton, closeButton;
  Checkbox lMediaClients;

  // Transcoding controls
  Checkbox      useTranscoding;
  CheckboxGroup bandwidthGroup;
  Checkbox[]    bandwidths;
  int           numberOfBandwidths;
  Label         bandwidthLabel;

  // Stream data
  Vector        streamName;
  Vector        streamEncode;
  Vector        streamDuration;
  Vector        streamSize;
  Vector        streamAvgBW;
  Checkbox[]    streamToggles;
  int           numberOfStreams;

  // Stream setup info
  Vector streamId;
  Vector streamHost;
  Vector streamPort;
  Vector streamPayload;
	
  // Column headers for stream data
  Label         streamHeadName1;
  Label         streamHeadEncode1;
  Label         streamHeadDuration1;
  Label         streamHeadSize1;
  Label         streamHeadAvgBW1;
  Label         streamHeadToggles1;
  Label         streamHeadName2;
  Label         streamHeadEncode2;
  Label         streamHeadDuration2;
  Label         streamHeadSize2;
  Label         streamHeadAvgBW2;
  Label         streamHeadToggles2;


  // Session descriptor controls
  Label         sessionDesc;
  Label         sessionTitle;
  Label         sessionName;

  Voyager.rtsp.Session playbackSession = new Session();
  Voyager.rtsp.Client  playbackClient = new TCPClient();
  String  playbackID;

  int serverSessionId;
  String audioHost = null;
  int audioPort = 0;
  String videoHost = null;
  int videoPort = 0;
  int stopped = 1;

  Frame frame_;
  VoyContext context_;

  // Master layout manager
  GridBagLayout masterLayout = new GridBagLayout( );

    Button dpButton_;
    DestPortData dpData_;
    boolean qHaveAudio , qHaveVideo , qHaveVr;
    String curDest_;
    String curPort_;

  public void init( VoyContext context , RtspURL url, Frame frame )
  {
    RTPFormats_ = new RTPFormatTable();

    url_ = url;
    context_ = context;
    frame_ = frame;

    dpData_ = new DestPortData( );
    dpData_.videoPort( context_.getVideoPort( ) );
    dpData_.audioPort( context_.getAudioPort( ) );
    dpData_.audioDest( context_.getAudioDest( ) );
    dpData_.videoDest( context_.getVideoDest( ) );
    dpButton_ = new Button( "Edit Destination/Port" );
    dpButton_.addActionListener(this);
    qHaveVideo = qHaveAudio = qHaveVr = false;
    curDest_ = new String( );
    curPort_ = new String( );

    playbackSession.setId(url_.getFile());

    initSession();

    streamName     = new Vector( );
    streamEncode   = new Vector( );
    streamDuration = new Vector( );
    streamSize     = new Vector( );
    streamAvgBW    = new Vector( );
    streamId = new Vector( );
    streamHost = new Vector( );
    streamPort = new Vector( );
    streamPayload = new Vector( );

    numberOfStreams = 0;
		
    this.setLayout( masterLayout );

    if( parseStreams( ) )
    {
	setupDescriptorControls( );

	setupPlaybackControls( );

	setupTranscodeControls( );

	setupStreamControls( );

	constrain( this , transcodePanel , 0 , 2 , 1 , 
		   1 , GridBagConstraints.HORIZONTAL ,
		   GridBagConstraints.WEST , 0.0 , 1.0 , 10 , 10 , 5 , 5 );
	constrain( this , streamListPanel , 0 , 1 , 1 , 
		   1 , GridBagConstraints.HORIZONTAL ,
		   GridBagConstraints.CENTER , 0.0 , 1.0 , 10 , 10 , 5 , 5 );
	constrain( this , sessionDescPanel , 0 , 0 , 1 , 
		   1 , GridBagConstraints.HORIZONTAL ,
		   GridBagConstraints.CENTER , 0.0 , 1.0 , 10 , 10 , 5 , 5 );
	constrain( this , vcrPanel , 0 , 3 , 1 , 
		   1 , GridBagConstraints.HORIZONTAL ,
		   GridBagConstraints.CENTER , 0.0 , 1.0 , 10 , 10 , 5 , 5 );

	constrain( this , dpButton_ , 0 , 4 , 1 ,
		   1 , GridBagConstraints.NONE ,
		   GridBagConstraints.CENTER , 0.0 , 1.0 , 10 , 10 , 5 , 5 );
	
	ScrollPane sp = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
	sp.add("Center", this);
	frame_.add("Center", sp);
	frame_.setSize(700, 700);
	frame_.show();
    }
    else
    {
	InfoDialog d = new InfoDialog(new Frame("Error"), "Error", "An error was encountered reading the media streams.  Perhaps this session is empty." );
	d.show();
	frame_.dispose();
    }
  }

  public void actionPerformed(ActionEvent e)
  {
    if(e.getActionCommand().equals("Play"))
      doPlayCommand();

    if(e.getActionCommand().equals("Stop"))
      doStopCommand();

    if(e.getActionCommand().equals("Quit"))
      doQuitCommand();

    if( e.getActionCommand( ).equals( "Edit Destination/Port" ) )
	doDestPortCmd( );
  }

    private void doDestPortCmd( )
    {
	DestPortUI d = new DestPortUI( frame_ , dpData_ , qHaveVideo ,
				       qHaveAudio , qHaveVr );
	Point loc = dpButton_.getLocation( );
	d.setLocation( loc );
	d.setVisible( true );
	if( d.dataChanged( ) )
	{
	    //System.out.println( "DPDATA is not the same!" );
	    if( context_.getCLaunched() )
		context_.killMediaClients();
	    lMediaClients.setState(true);
	}
    }

  private void doPlayCommand( )
  {
    int i;
    stopped = 0;
    playButton.disable( );
    dpButton_.disable( );
    boolean setupErr = false;
    for(i = 0; !setupErr && i < numberOfStreams; i++ )
      {
	if( streamToggles[ i ].getState( ) )
	  {
	    setupErr = sendSetup( i );
	  }
	streamToggles[ i ].disable( );
      }
      
    if( !setupErr )
    {

    stopButton.enable( );

    useTranscoding.disable( );

    try {
	if(lMediaClients.getState() && !context_.getCLaunched())
	{
	    String audioDest = new String( );
	    String videoDest = new String( );

	    InetAddress inetaddr = InetAddress.getByName(dpData_.audioDest( ));
	    audioDest = inetaddr.isMulticastAddress( )? dpData_.audioDest( ) :
		                                        context_.serverHost( );

	    inetaddr = InetAddress.getByName(dpData_.videoDest( ));
	    videoDest = inetaddr.isMulticastAddress( )? dpData_.videoDest( ) :
		                                        context_.serverHost( );

	    context_.launchMediaClients( audioDest , videoDest ,
					Integer.toString(dpData_.audioPort()),
					Integer.toString(dpData_.videoPort()));
	    lMediaClients.setState(false);
	}
    } catch( UnknownHostException e ) {
	// put something here inna minute
    }

    playbackClient.playSession( playbackSession.id() , serverSessionId );
    //context_.showStatus( "returned from play" );
    }
  }

  private void doStopCommand()
  {
    int i;
    for( i=0 ; i<numberOfStreams ; i++ )
      {
	if( streamToggles[ i ].getState( ) )
	  sendTeardown( i );
	streamToggles[ i ].enable( );
      }
    playButton.enable( );
    stopButton.disable( );
    dpButton_.enable( );
    useTranscoding.enable( );
    serverSessionId = -1;
    stopped = 1;
  }

  private void doQuitCommand()
  {
    context_.removeClosable(this);
    int i;
    if(stopped == 0)
      {
	for( i=0 ; i<numberOfStreams ; i++ )
	  {
	    if( streamToggles[ i ].getState( ) )
	      sendTeardown( i );
	    streamToggles[ i ].enable( );
	  }
      }

    if( context_.getCLaunched() )
	context_.killMediaClients();
    
    frame_.dispose();
    //    java.lang.System.exit(0);
  }

  public void constrain( Container container , Component component, 
			 int grid_x, int grid_y, 
			 int grid_width, int grid_height, int fill, 
			 int anchor, double weight_x, double weight_y, 
			 int top, int left, int bottom, int right )
  {
    GridBagConstraints c = new GridBagConstraints( ) ;
    c.gridx = grid_x;
    c.gridy = grid_y;
    c.gridwidth = grid_width;
    c.gridheight = grid_height;
    c.fill = fill;
    c.anchor = anchor;
    c.weightx = weight_x;
    c.weighty = weight_y;

    if( top+bottom+left+right > 0 )
      c.insets = new Insets( top , left , bottom , right );

    ( (GridBagLayout)container.getLayout( ) ).setConstraints( component , c );

    container.add( component );
  }

  public void constrain( Container container , Component component , int grid_x ,
			 int grid_y , int grid_width , int grid_height )
  {
    constrain( container , component , grid_x , grid_y , grid_width , grid_height ,
	       GridBagConstraints.NONE , GridBagConstraints.NORTHWEST , 0.0 ,
	       0.0 , 0 , 0 , 0 , 0 );
  }

  public void constrain( Container container , Component component , 
			 int grid_x , int grid_y , int grid_width , 
			 int grid_height , int top , int left ,
			 int bottom , int right )
  {
    constrain( container , component , grid_x , grid_y , grid_width , grid_height ,
	       GridBagConstraints.NONE , GridBagConstraints.NORTHWEST , 0.0 ,
	       0.0 , top , left , bottom , right );
  }

  public void setupTranscodeControls( )
  {

    int i;
    
    numberOfBandwidths = 4;

    useTranscoding  = new Checkbox( "Transcode to h.261" , null , false );
    useTranscoding.setFont(new Font("Helvetica", Font.BOLD, 12));

    bandwidthGroup  = new CheckboxGroup( );
    bandwidths      = new Checkbox[ numberOfBandwidths ];
    bandwidths[0] = new Checkbox( "1Mbps" , bandwidthGroup , false );
    bandwidths[0].setFont(new Font("Helvetica", Font.BOLD, 12));

    bandwidths[1] = new Checkbox( "256Kbps" , bandwidthGroup , false );
    bandwidths[1].setFont(new Font("Helvetica", Font.BOLD, 12));

    bandwidths[2] = new Checkbox( "128Kbps" , bandwidthGroup , true );
    bandwidths[2].setFont(new Font("Helvetica", Font.BOLD, 12));

    bandwidths[3] = new Checkbox( "64Kbps" , bandwidthGroup , false );
    bandwidths[3].setFont(new Font("Helvetica", Font.BOLD, 12));

    bandwidthLabel  = new Label( "Limit bandwidth to:" );
    bandwidthLabel.setFont(new Font("Helvetica", Font.BOLD, 12));


    for( i = 0 ; i < numberOfBandwidths ; i++ )
      {
	bandwidths[ i ].disable( );
      }
    bandwidthLabel.disable( );

    transcodePanel = new Panel( );
    transcodePanel.setLayout( masterLayout );

    constrain( transcodePanel , useTranscoding , 0 , 0 , 1 , 1 );
    constrain( transcodePanel , bandwidthLabel , 0 , 1 , 1 , 1 );

    for( i = 0 ; i < numberOfBandwidths ; i++ )
      {
	constrain( transcodePanel , bandwidths[ i ] , i+1 , 1 , 1 , 1 );
      }
  }

  public void setupStreamControls( )
  {
    int i;

    streamToggles = new Checkbox[ numberOfStreams ];
    for( i=0 ; i<numberOfStreams ; i++ )
      streamToggles[ i ] = new Checkbox( null , null , true );

    streamListPanel = new Panel( );
    streamListPanel.setLayout( masterLayout );

    streamHeadName1 = new Label( " " );
    streamHeadEncode1 = new Label( "Encoding" );
    streamHeadEncode1.setFont(new Font("Helvetica", Font.BOLD, 12));
    streamHeadDuration1 = new Label( "Duration" );
    streamHeadDuration1.setFont(new Font("Helvetica", Font.BOLD, 12));
    streamHeadSize1 = new Label( "Size" );
    streamHeadSize1.setFont(new Font("Helvetica", Font.BOLD, 12));
    streamHeadAvgBW1 = new Label( "AvgBW" );
    streamHeadAvgBW1.setFont(new Font("Helvetica", Font.BOLD, 12));
    streamHeadToggles1 = new Label( "Play" );
    streamHeadToggles1.setFont(new Font("Helvetica", Font.BOLD, 12));

    constrain( streamListPanel , streamHeadName1     , 0 , 0 , 1 , 1 ,
	       GridBagConstraints.NONE , GridBagConstraints.CENTER , 0.0 , 
	       1.0 , 0 , 0 , 0 , 0 );
    constrain( streamListPanel , streamHeadEncode1   , 1 , 0 , 1 , 1 ,
	       GridBagConstraints.NONE , GridBagConstraints.CENTER , 0.0 , 
	       1.0 , 0 , 0 , 0 , 0 );
    constrain( streamListPanel , streamHeadDuration1 , 2 , 0 , 1 , 1 ,
	       GridBagConstraints.NONE , GridBagConstraints.CENTER , 0.0 , 
	       1.0 , 0 , 0 , 0 , 0 );
    constrain( streamListPanel , streamHeadSize1     , 3 , 0 , 1 , 1 , 
	       GridBagConstraints.NONE , GridBagConstraints.EAST , 0.0 , 
	       1.0 , 0 , 0 , 0 , 0 );
    constrain( streamListPanel , streamHeadAvgBW1    , 4 , 0 , 1 , 1 , 
	       GridBagConstraints.NONE , GridBagConstraints.EAST , 0.0 , 
	       1.0 , 0 , 0 , 0 , 0 );
    constrain( streamListPanel , streamHeadToggles1  , 5 , 0 , 1 , 1 ,
	       GridBagConstraints.NONE , GridBagConstraints.CENTER , 0.0 , 
	       1.0 , 0 , 0 , 0 , 0 );

    streamHeadName2 = new Label( "User" );
    streamHeadName2.setFont(new Font("Helvetica", Font.PLAIN, 12));
    streamHeadEncode2 = new Label( "(Type)" );
    streamHeadEncode2.setFont(new Font("Helvetica", Font.PLAIN, 12));
    streamHeadDuration2 = new Label( "(min:sec)" );
    streamHeadDuration2.setFont(new Font("Helvetica", Font.PLAIN, 12));
    streamHeadSize2 = new Label( "(KB)" );
    streamHeadSize2.setFont(new Font("Helvetica", Font.PLAIN, 12));
    streamHeadAvgBW2 = new Label( "(Kbps)" );
    streamHeadAvgBW2.setFont(new Font("Helvetica", Font.PLAIN, 12));
    streamHeadToggles2 = new Label( "Stream" );
    streamHeadToggles2.setFont(new Font("Helvetica", Font.PLAIN, 12));

    constrain( streamListPanel , streamHeadName2     , 0 , 1 , 1 , 1 ,
	       GridBagConstraints.NONE , GridBagConstraints.CENTER , 0.0 , 
	       1.0 , 1 , 1 , 1 , 1 );
    constrain( streamListPanel , streamHeadEncode2   , 1 , 1 , 1 , 1 ,
	       GridBagConstraints.NONE , GridBagConstraints.CENTER , 0.0 , 
	       1.0 , 1 , 1 , 1 , 1 );
    constrain( streamListPanel , streamHeadDuration2 , 2 , 1 , 1 , 1 ,
	       GridBagConstraints.NONE , GridBagConstraints.CENTER , 0.0 , 
	       1.0 , 1 , 1 , 1 , 1 );
    constrain( streamListPanel , streamHeadSize2     , 3 , 1 , 1 , 1 , 
	       GridBagConstraints.NONE , GridBagConstraints.EAST , 0.0 , 
	       1.0 , 1 , 1 , 1 , 1 );
    constrain( streamListPanel , streamHeadAvgBW2    , 4 , 1 , 1 , 1  , 
	       GridBagConstraints.NONE , GridBagConstraints.EAST , 0.0 , 
	       1.0 , 1 , 1 , 1 , 1 );
    constrain( streamListPanel , streamHeadToggles2  , 5 , 1 , 1 , 1 ,
	       GridBagConstraints.NONE , GridBagConstraints.CENTER , 0.0 , 
	       1.0 , 1 , 1 , 1 , 1 );

    for( i=0 ; i<numberOfStreams ; i++ )
      {
	constrain( streamListPanel , (Label)streamName.elementAt( i ) , 0 , 
		   i+2 , 1 , 1 , GridBagConstraints.NONE , 
		   GridBagConstraints.WEST , 0.0 , 1.0 , 1 , 1 , 1 , 1 );
	constrain( streamListPanel , (Label)streamEncode.elementAt( i ) , 1 , 
		   i+2 , 1 , 1 , GridBagConstraints.NONE , 
		   GridBagConstraints.WEST , 0.0 , 1.0 , 1 , 1 , 1 , 1 );
	constrain( streamListPanel , (Label)streamDuration.elementAt( i ) , 2 , 
		   i+2 , 1 , 1 , GridBagConstraints.NONE , 
		   GridBagConstraints.CENTER , 0.0 , 1.0 , 1 , 1 , 1 , 1 );
	constrain( streamListPanel , (Label)streamSize.elementAt( i ) , 3 , 
		   i+2 , 1 , 1 , GridBagConstraints.NONE , 
		   GridBagConstraints.EAST , 0.0 , 1.0 , 1 , 1 , 1 , 1 );
	constrain( streamListPanel , (Label)streamAvgBW.elementAt( i ) , 4 , 
		   i+2 , 1 , 1 , GridBagConstraints.NONE , 
		   GridBagConstraints.EAST , 0.0 , 1.0 , 1 , 1 , 1 , 1 );
	constrain( streamListPanel , streamToggles[ i ]  , 5 , i+2 , 1 , 1 , 
		   GridBagConstraints.NONE , GridBagConstraints.CENTER , 0.0 ,
		   1.0 , 1 , 1 , 1 , 1 );
      }
  }

  public void setupDescriptorControls( )
  {
    sessionTitle = new Label( "Voyager Playback Manager" );
    sessionTitle.setFont(new Font("Helvetica", Font.BOLD, 18));

    sessionDesc = new Label( playbackSession.description( ) );
    sessionDesc.setFont(new Font("Helvetica", Font.BOLD, 12));
    sessionName = new Label( playbackSession.name( ) );
    sessionName.setFont(new Font("Helvetica", Font.BOLD, 14));

    sessionDescPanel = new Panel( );
    sessionDescPanel.setLayout( masterLayout );
    constrain( sessionDescPanel , sessionTitle , 0 , 0 , 1 , 1 );
    constrain( sessionDescPanel , sessionDesc , 0 , 1 , 1, 1 ,
	       GridBagConstraints.NONE , GridBagConstraints.CENTER , 
	       0.0 , 1.0 , 1 , 1 , 1 , 1 );
    constrain( sessionDescPanel , sessionName , 0 , 2 , 1, 1 ,
	       GridBagConstraints.NONE , GridBagConstraints.CENTER , 
	       0.0 , 1.0 , 1 , 1 , 1 , 1 );

  }

  public void setupPlaybackControls( )
  {
    playButton = new Button( "Play" );
    playButton.setFont(new Font("Helvetica", Font.BOLD, 12));
    playButton.setSize( 57 , 26 );
    playButton.addActionListener(this);

    stopButton = new Button( "Stop" );
    stopButton.setFont(new Font("Helvetica", Font.BOLD, 12));
    stopButton.disable( );
    stopButton.setSize( 57 , 26 );
    stopButton.addActionListener(this);
    
    quitSessionButton = new Button( "Quit" );
    quitSessionButton.setFont(new Font("Helvetica", Font.BOLD, 12));
    quitSessionButton.setSize( 57 , 26 );
    quitSessionButton.addActionListener(this);

    lMediaClients = new Checkbox("Launch Media Clients", null, true);
    lMediaClients.setFont(new Font("Helvetica", Font.BOLD, 12));

    vcrPanel = new Panel( );
    vcrPanel.setLayout( masterLayout );
    
    constrain( vcrPanel , playButton , 1 , 1 , 1 , 1 ) ;
    constrain( vcrPanel , stopButton , 2 , 1 , 1 , 1 );
    constrain( vcrPanel , quitSessionButton , 3 , 1 , 1 , 1 );
    constrain( vcrPanel , lMediaClients , 0 , 1, 1, 1);
  }

  public boolean parseStreams( )
  {
    boolean bSuccess = false;
    DescribeStreamResult descStreamR;

    serverSessionId = -1;
    numberOfStreams = 0;
    Enumeration eStreams = playbackSession.getStreams( );
    while( eStreams.hasMoreElements( ) )
      {
	numberOfStreams++;
	String streamStr = (String) eStreams.nextElement( );

	descStreamR = playbackClient.describeStream( playbackSession.id(), streamStr );
	if( descStreamR.success( ) )
	  {
	    String value;

	    if( !bSuccess )
	      bSuccess = true;

	    Stream stream = descStreamR.stream( );
	    value = stream.getParameter( "sdes_cname" );
	    if( value == null )
	      value = "unknown";
	    streamName.addElement( new Label( value ) );
	    value = stream.getParameter( "packet_types" );
	    String encoding = (String)RTPFormats_.get( value );
	    streamEncode.addElement( new Label( encoding ) );
	    int pType = Integer.parseInt( value );
	    saveSetupInfo( pType , streamStr );

	    value = stream.getParameter( "duration" );
	    Float duration = Float.valueOf( value );
	    int min = (int)( duration.doubleValue( ) / 60 );
	    int sec = (int)( duration.doubleValue( ) % 60 );
	    value = Integer.toString( min ) + ":" + Integer.toString( sec );
	    streamDuration.addElement( new Label( value ) );

	    int size = 0;
	    String sizeValue;
	    String bwValue;
	    value = stream.getParameter( "bytes" );
	    if( value != null )
	      {
		NumberFormat twoDecPlaces = NumberFormat.getInstance( );
		twoDecPlaces.setMaximumFractionDigits( 2 );
		twoDecPlaces.setMinimumFractionDigits( 2 );
		    
		size = Integer.parseInt( value );
		double bw = 8 * size / duration.doubleValue( );

		float kSize = (float)( size / 1024. );
		kSize = (float)( (int)( kSize * 100 ) / 100. );
		sizeValue = twoDecPlaces.format( kSize );

		float kBW = (float)( bw / 1024. );
		kBW = (float)( (int)( kBW * 100 ) / 100. );
		bwValue = twoDecPlaces.format( kBW );
	      }
	    else
	      {
		sizeValue = "unknown";
		bwValue = "unknown";
	      }
	    streamSize.addElement( new Label( sizeValue ) );
	    streamAvgBW.addElement( new Label( bwValue ) );
	  }
      }
		
    return bSuccess;
  }

  public void saveSetupInfo( int pType , String stream )
  {
    switch( pType )
      {
      case 0:
      case 1:
      case 2:
      case 3:
      case 5:
      case 6:
      case 7:
      case 8:
      case 9:
      case 10:
      case 11:
      case 14:
      case 15:
	qHaveAudio = true;
	break;
      case 25:
      case 26:
      case 28:
      case 31:
      case 32:
	qHaveVideo = true;
	break;
      case 95:
      case 96:
	qHaveVr = true;
	break;
      }
    streamId.addElement( new String( stream ) );
    streamPayload.addElement( new Integer(pType) );
  }


  public void setCurDestPort( int i )
  {
      Integer pType = (Integer)streamPayload.elementAt(i);

    switch( pType.intValue( ) )
      {
      case 0:
      case 1:
      case 2:
      case 3:
      case 5:
      case 6:
      case 7:
      case 8:
      case 9:
      case 10:
      case 11:
      case 14:
      case 15:
	curDest_ = dpData_.audioDest( );
	curPort_ = Integer.toString( dpData_.audioPort( ) );
	break;
      case 25:
      case 26:
      case 28:
      case 31:
      case 32:
	curDest_ = dpData_.videoDest( );
	curPort_ = Integer.toString( dpData_.videoPort( ) );
	break;
      case 95:
      case 96:
	curDest_ = dpData_.vrDest( );
	curPort_ = Integer.toString( dpData_.vrPort( ) );
	break;
      }
  }


  public boolean sendSetup( int i )
  {
      boolean setupErr = false;
      //context_.showStatus( "Sending  a setup" );
      setCurDestPort( i );
    
      try {
	  InetAddress inetDest = InetAddress.getByName( curDest_ );
	  String ttlStr = new String( "" );
	  if( inetDest.isMulticastAddress( ) )
	      ttlStr = "ttl="+ Integer.toString( dpData_.ttl( ) ) + ";";
		
	  //context_.showStatus( "stream " + (String)streamId.elementAt( i ) + 
	  //	       " assigned to " + curDest_ + " and port " + 
	  //	       curPort_ + " for type " + 
	  //	       ((Integer)streamPayload.elementAt(i)).toString( ) );

	  SetupStreamResult src = playbackClient.setupStream( playbackSession.id() ,
							      (String)streamId.elementAt( i ) ,
							      "RTP/AVP;destination=" + curDest_ + ";port=" + 
							      curPort_ + ";" + ttlStr +
							      "mode=\"PLAY\"" , serverSessionId );

      if( !src.success( ) )
      {
	    final Dialog d = new Dialog( frame_, "Playback Setup Failed", true );

	    Point loc = frame_.getLocation( );
	    d.setBounds( loc.x , loc.y , 40 , 40 );

	    d.add("Center", new Label("Error on playback setup request:  Perhaps an address/port pair is invalid or the port is currently in use."));

	    Button ok;
	    d.add("South", ok = new Button("OK"));

	    ok.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    d.hide();
		}});
	    d.setLocation( loc );
	    d.pack();
	    d.show();
	    d.dispose();
	    frame_.dispose( );
	    if( context_.getCLaunched( ) )
	        context_.killMediaClients( );
	    setupErr = true;
      }
      else
      {
	  if( serverSessionId == -1 )
	      {
		  serverSessionId = src.rtspSessionId( );
		  //context_.showStatus( "Got session id " + serverSessionId );
	      }
      }
      } catch( UnknownHostException e ) {
	  InfoDialog d = new InfoDialog( new Frame( "Error" ) , "Error" ,
					 "Unknown host " + curDest_ );
	  d.show( );
	  doQuitCommand( );
	    setupErr = true;
      }
    return setupErr;
  }

  public void sendTeardown( int i )
  {
      //context_.showStatus( "Sending  a teardown" );

    TeardownStreamResult src = playbackClient.teardownStream( playbackSession.id() ,
							      (String)streamId.elementAt( i ) , 
							      serverSessionId );
    
    //context_.showStatus( "After teardown" );
  }

  private void initSession( )
  {

      //context_.showStatus( "HOST: " + url_.getHost() + " PORT: " + url_.getPort() );
    context_.registerClosable(this);

    ConnectResult connectR = playbackClient.connect( url_.getHost() , url_.getPort() );

    if( connectR.success( ) )
      {
	DescribeSessionResult descSessionR = 
	  playbackClient.describeSession( playbackSession.id() ) ;

	if( descSessionR.success( ) )
	  {
	    playbackSession = descSessionR.session( );
	    playbackSession.dump( );
	  }
	else
	  {
	    InfoDialog d = new InfoDialog(new Frame("Error"), "Error", 
		  "An error was encountered retrieving session information" );
	    d.show( );
	  }
      }
    else
      {
	InfoDialog d = new InfoDialog(new Frame("Error"), "Error", 
	     "An error was encountered creating a connection to the session.  The Voyager Server may not be running." );
	d.show();
      }
  }

  public void shutdown() {
    doQuitCommand();
  }
}

