package Voyager.rtsp.clients;

import java.awt.*;
import java.awt.event.*;
import Voyager.rtsp.*;
import Voyager.util.*;

/*
 * RecordUI is a panel; it has a CardLayout which contains each
 * of the various UI panels. It handles the cycling of the
 * panels when the Next> button is pressed on each.
 */


public class RecordUI extends Panel implements Closable
{
    AddressSelectorPanel addressPanel_;
    SessionInfoPanel sessionInfoPanel_;
    CreatingSessionPanel creatingSessionPanel_;
    ControlPanel controlPanel_;

    RecordController controller_;

    Frame frame_;
    CardLayout layout_;
    VoyContext context_;

    public RecordUI(RecordController c , Frame frame, VoyContext context )
    {
	frame_ = frame;
	context_ = context;
	context_.registerClosable(this);
	setupFrame();
	createPanels();

	layout_.show(this, "session");
	controller_ = c;
    }

    void setupFrame()
    {
	layout_ = new CardLayout();
	setLayout(layout_);
    }

    void createPanels()
    {
	addressPanel_ = new AddressSelectorPanel(this);
	sessionInfoPanel_ = new SessionInfoPanel(this);
	creatingSessionPanel_ = new CreatingSessionPanel(this);
	controlPanel_ = new ControlPanel(this);

	add("session", sessionInfoPanel_);
	add("address", addressPanel_);
	add("wait", creatingSessionPanel_);
	add("control", controlPanel_);

    }

    void addressPanelBack()
    {
	//
	// Whoops, waffling user
	//
	layout_.show(this, "session");
    }

    void addressPanelDone()
    {
	int type = addressPanel_.getSelectionType();

	switch (type)
	{
	case AddressSelectorPanel.AUTOMATIC:
	    controller_.setAutomaticAddress();
	    break;
	    
	case AddressSelectorPanel.MANUAL:
	    controller_.setManualAddress(addressPanel_.getManualAddress());
	    break;

	case AddressSelectorPanel.FLBONE:
	    controller_.setFLAddress( );
	    break;

	case AddressSelectorPanel.ACTINIDES:
	    controller_.setActinidesAddress( );
	    break;
	}

	//
	// At this point, we have all the info we need.
	// Switch to the "please wait panel" and send the controller on its way
	//
	layout_.show(this, "wait");

	boolean success = false;
	try {
	    controller_.commit();
	    success = true;
	}
	catch (RTSPFailedException ex)
	{
	    final Dialog d = new Dialog( frame_, "Record Setup Failed", true );

	    Point loc = frame_.getLocation( );
	    d.setBounds( loc.x , loc.y , 40 , 40 );

	    d.add("Center", new Label("Error on record setup request:  Perhaps the address/port pair is invalid or is currently being recorded by another Voyager client"));

	    Button ok;
	    d.add("South", ok = new Button("OK"));

	    ok.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    d.hide();
		}});
	    d.setLocation( loc );
	    d.pack();
	    d.show();
	    d.dispose();
	    frame_.dispose( );
	    context_.removeClosable( this );
	}

	//System.out.println("done with connect, success=" + success);
	if (!success)
	{
	    layout_.show(this, "address");
	    return;
	}
	
	//
	// Okay, he's done. Go on to the control panel.
	//

	layout_.show(this, "control");
    }

    void sessionInfoPanelDone()
    {
	String name = sessionInfoPanel_.getSessionName();
	String desc = sessionInfoPanel_.getSessionDescription();

	//System.out.println("Got info panel done. Name is " + name);
	//System.out.println("Description:");
	//System.out.println(desc);

	controller_.setSessionInfo(name, desc);

	//
	// Move to the next card, the address panel
	//

	layout_.show(this, "address");
    }

    void controlPanelDone()
    {
	//System.out.println("Got control panel done");
	controller_.teardownSession( );
	frame_.dispose( );
	context_.removeClosable(this);
    }

    void  controlPanelRecord( )
    {
	//System.out.println( "Record request" );
	try {
	    controller_.recordSession( );
	} catch( RTSPFailedException ex ) {
	    final Dialog d = new Dialog( frame_ , "Record Failed" , true );
	    d.add( "Center" , new Label( "Error on record request" ) );
	    Button ok;
	    d.add( "South" , ok=new Button("OK") );
	    ok.addActionListener( new ActionListener( ) {
		public void actionPerformed( ActionEvent e) {
		    d.hide( );
		}} );
	    Point loc = frame_.getLocation( );
	    d.setLocation( loc );
	    d.pack( );
	    d.show( );
	    d.dispose( );
	    frame_.dispose( );
	    context_.removeClosable(this);
	}
    }

  public void shutdown() {
    //what do we need to do here?
  }
}
