package Voyager.util;

import javax.media.protocol.DataSource;
import javax.media.rtp.RTPControl;
import javax.media.rtp.BaseEncodingInfo;
import javax.media.rtp.RTPSessionManager.RTPSessionManager;

/**
 * This is a utility class which configures a RTP datasource with
 * dynamic payload information using the interface defined in
 * javax.media.rtp.RTPControl.The main method is the Init() method which
 * will take in a datasource as an argument. Currently, the datasource
 * is configured for dynamic payload types used by ShowmeTvT, which
 * tranmits DVI compressed audio at 11.025, 22.05 and 44.1 KHz at
 * dynamic RTP payloads 16,17 and 18 respectively. The JMF RTP handlers
 * are capable of handling these content types. The RTP datasource must
 * be configured with dynamic payload information prior to attempting to
 * playback these streams.
 */
public class EncodingUtil {
    private static final String rtpcontrol = "javax.media.rtp.RTPControl";

    
    public static void Init(DataSource ds){
	RTPControl control = (RTPControl)ds.getControl(rtpcontrol);
	if (control == null){
	    System.out.println("The datasource " + ds +
			       "\n does not support the RTPCOntrol interface..." +
			       "\nexiting from EncodingUtil.Init()");
	    return;
	}
	Init(control);
    }
    public static void Init(RTPSessionManager mgr){
	BaseEncodingInfo info = null;
	info = new BaseEncodingInfo(16,"dvi",11025,
				    1,4,
				    null);
	mgr.addEncoding(info);
	// dvi at 22050 Hz
	info = new BaseEncodingInfo(17,"dvi",22050,
				    1,4,
				    null);
	mgr.addEncoding(info);
	// dvi at 44100 Hz
	info = new BaseEncodingInfo(18,"dvi",44100,
				    1,4,
				    null);
	mgr.addEncoding(info);
    }
    
    public static  void Init(RTPControl control){
	BaseEncodingInfo info = null;
	if (control == null)
	    return;
	// since we have a handle over the RTPControl interface, now
	// add our 3 dynamic payload types for DVI. These are
	// PT         encoding      audio/video    clock rate    channels
	//             name          (A/V)          (Hz)          (audio)
	// 16          DVI4          A              11025          1
	// 17          DVI4          A              22050          1
	// 18          DVI4          A              44100          1
	// for each of the payloads, we will create a
	// BaseEncodingInfo (defined in
	// javax.media.rtp.BaseEncodingInfo) and add this to the
	// RTPControl interface. Note: as explained in the RTPControl
	// inteface, addEncoding() can be called repeatedly in order
	// to enter dynamic payload encoding information.
	// DVI at 11025Hz
	info = new BaseEncodingInfo(16,"dvi",11025,
				    1,4,
				    null);
	control.addEncoding(info);
	// dvi at 22050 Hz
	info = new BaseEncodingInfo(17,"dvi",22050,
				    1,4,
				    null);
	control.addEncoding(info);
	// dvi at 44100 Hz
	info = new BaseEncodingInfo(18,"dvi",44100,
				    1,4,
				    null);
	control.addEncoding(info);
    }
}
