/*
 * @(#)RTPPlayer.java	1.1 97/06/26  
 * 
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package Voyager.util;

import GUI.*;
import javax.media.rtp.RTPSessionManager.*;
import com.sun.media.rtp.RTPPSM.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.lang.String;
import javax.media.*;
import javax.media.protocol.*;
import javax.media.rtp.*;
import com.sun.media.*;
import java.util.Vector;
import java.io.IOException;
import com.sun.media.ui.*;
// this sample example illustrates use of the RTPSession Manager API.
// The sample allows you to create one audio and one video player to
// playback broadcast, multicast and unicast RTP sessions. Video and
// Audio monitors are also provided to monitor each of the RTP
// sessions.A synopsis of the methods in this example follows:
// RTPPlayerApp constructor will basically create the GUI for entering
// data on the RTPSession. The user can also enter the RTPSession
// address from the command line by using
// java RTPPlayerApp -audio <session address> -video <session address>
// method actionPerfomed() will take in AWT user events and make
// appropriate calls for creating the RTP player.
// method startSessionManager() actually contains RTPAPI calls to
// initiate the RTP session manager and register as a
// RTPSessionListener.
// finally, update() methods will process all the RTPEvents received
// from the RTPSessionManager for listeners that have registered with the RTPSM
public class RTPPlayerApp extends Frame implements
ActionListener, RTPRecvStreamListener {

    // audio and video URL textfields
    TextField urlvideo;
    TextField urlaudio;
    // AWT buttons for displaying audio and video RTP Monitors
    Button videobutton;
    Button audiobutton;
    // variables to hold the URL fields
    char[] nm;
    boolean baddress ;
    boolean bport;
    boolean bmedia;
    boolean bttl;
    InetAddress destaddr;
    String address;
    String portstr;
    String ttlstr;
    String media;
    int offset = 0;
    int ttl = 1;
    
    
    // audio and video RTPSessionManagers
    RTPSessionManager videomgr = null;
    RTPSessionManager audiomgr = null;

    GridBagLayout gridbag;
    GridBagConstraints c;
    // the video and audio RTP participantlist
    ParticipantListWindow videogui = null;
    ParticipantListWindow audiogui = null;

    /**
     * This is the main() method and deals with taking in session
     * parameters for RTPPlayerApp from the command line.
     */
    public static void main(String[] args){
	// this is to take in parameters of the session from the command line.
	String videoAdd = null, audioAdd = null;
	int nArgs = args.length;
	int i = 0;
	while (i < nArgs - 1) {
	    if (args[i].indexOf("-video") == 0 && i < nArgs - 1) {
		videoAdd = "rtp://" + args[i+1] + "/video/";
		i++;
	    } else if (args[i].indexOf("-audio") == 0 && i < nArgs - 1) {
		audioAdd = "rtp://" + args[i+1] + "/audio/" ;
		i++;
	    }
	    i++;
	}

	new RTPPlayerApp(videoAdd, audioAdd);
    }

    /**
     * Constructor of RTPPlayerApp that will bring up the GUI for the
     * application and provide a default RTP session string.
     */
    public RTPPlayerApp(String vAdd, String aAdd) {
	super("JMF RTP Player");
	setFont(new Font("Helvetica", Font.PLAIN, 14));
	// GridBagLayout for GUI controls. The GUI will consist of 2
	// textfields and 2 buttons for displaying the RTP monitors of
	// each of the RTPSM
	gridbag = new GridBagLayout();
	c = new GridBagConstraints();
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.NORTHWEST;
	c.weightx = 1.0;
	c.weighty = 1.0;
	c.gridwidth = GridBagConstraints.REMAINDER;
	setLayout(gridbag);
	// Video URL TextField
	urlvideo = new TextField(50);
	if (vAdd != null)
	    urlvideo.setText(vAdd);
	else
	    urlvideo.setText("rtp://224.144.251.104/49152/1/video/");
	gridbag.setConstraints(urlvideo,c);
	add(urlvideo);
	// Video RTP Monitor button
	videobutton = new Button("Video RTP Monitor");
	gridbag.setConstraints(videobutton,c);
	videobutton.setEnabled(false);
	add(videobutton);
	// Audio URL TextField
	urlaudio = new TextField(50);
	if (aAdd != null)
	    urlaudio.setText(aAdd);
	else
	    urlaudio.setText("rtp://224.144.251.104/49150/1/audio/");
	gridbag.setConstraints(urlaudio,c);
	add(urlaudio);
 	// Audio RTP Monitor button
	audiobutton = new Button("Audio RTP Monitor");
	gridbag.setConstraints(audiobutton,c);
	audiobutton.setEnabled(false);
	add(audiobutton);
	// add yourself as listeners for the video and audio
	// textfields and buttons
	urlvideo.addActionListener(this);
	urlaudio.addActionListener(this);
	videobutton.addActionListener(this);
	audiobutton.addActionListener(this);
	
	pack();
	show();
	// code for closing the window
	this.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		String reason = "Shutdown JMF Player";
		if (videomgr != null)
		    videomgr.closeSession(reason);
		if (audiomgr != null)
		    audiomgr.closeSession(reason);
		System.exit(0);
	    }
	});
	
    }// end of constructor




    // this method create a new RTPSessionMgr and adds this application
    // as a RTPRecvStreamListener before calling initSession() and startSession()
    private RTPSessionMgr StartSessionManager(String destaddrstr,
					      int port,
					      String media,
					      int ttl){
	RTPSessionMgr mymgr = null;
	// create the new RTPSessionManager
	mymgr = new RTPSessionMgr();
	if (mymgr == null)
	    return null;
	// keep a handle over the video or audio manager as the case maybe
	if (media.equals("video"))
	    videomgr = mymgr;
	if (media.equals("audio"))
	    audiomgr = mymgr;
	// add yourself as a RecvStreamListener to listen to
	// RecvStreamEvents for RTPRecvStreams of this session
	mymgr.addRecvStreamListener(this);
	// add dynamic payload information to the RTPSM.
	if (media.equals("audio"))	
	    EncodingUtil.Init((RTPSessionManager)mymgr);
	
	// ask RTPSM to generate the local participants CNAME
	String cname = mymgr.generateCNAME();

	// create our local RTPSession Address
	RTPSessionAddress localaddr = new RTPSessionAddress();
	try{
	    destaddr = InetAddress.getByName(destaddrstr);
	}catch (UnknownHostException e){
	    System.out.println("inetaddress " + e.getMessage());
	}    
	RTPSessionAddress sessaddr = new RTPSessionAddress(destaddr,
							   port,
							   destaddr,
							   port +1);
	// we send the email, name, cname and tool in the SDES items
	RTCPSourceDescription[] userdesclist = new RTCPSourceDescription[4];
	int i;
	for(i=0; i< userdesclist.length;i++){
	    if (i == 0){
		userdesclist[i] = new
	      RTCPSourceDescription(RTCPSourceDescription.SOURCE_DESC_EMAIL,
					  "jmf-user@sun.com",
					  1,
					  false);
		continue;
	    }
	    if (i == 1){
		userdesclist[i] = new
	      RTCPSourceDescription(RTCPSourceDescription.SOURCE_DESC_NAME,
					  System.getProperty("user.name"),
					  1,
					  false);
		continue;
	    }
	    if ( i == 2){
	    	userdesclist[i] = new 
		RTCPSourceDescription(RTCPSourceDescription.SOURCE_DESC_CNAME,
				      cname,
				      1,
				      false);
		continue;
	    }
	    if (i == 3){ 
		userdesclist[i] = new
	    RTCPSourceDescription(RTCPSourceDescription.SOURCE_DESC_TOOL,
				  "JMF RTP Player v1.0",
				  1,
				  false);
		continue;
	    }
	}// end of for
	
	// call initSession() and startSession() on the RTPSM.
	try{
	    mymgr.initSession(localaddr,
			      userdesclist,
			      0.05,
			      0.25);
	    mymgr.startSession(sessaddr,ttl,null);
	}catch (RTPSessionManagerException e){
	    System.out.println("InitSession(): StartSession():Exception " +
			       e.getMessage());
	    return null;
	}catch (IOException e){
	    System.out.println("StartSession : IOException " + e.getMessage());
	    return null;
	}
	
	return mymgr;
    }

    
    // Since this class implements the RTPSessionListener interface,
    // it must implement the RTPSessionUpdate() method
    public void update(RTPRecvStreamEvent event){
	
	Player newplayer = null;
	PlayerWindow playerWindow = null;

	// find the sourceRTPSM for this event
	RTPSessionManager source =
	    (RTPSessionManager)event.getSource();
	
	// event is a BYE Event. Print this out
	if (event instanceof ByeEvent)
	    System.out.println("ByeEvent");

	// create a new player if a new recvstream is detected
	if (event instanceof NewRecvStreamEvent){
	    try{
		// get a handle over the RTPRecvStream
		RTPRecvStream stream =
		    ((NewRecvStreamEvent)event).getRecvStream();
		// get a handle over the RecvStream datasource
		DataSource dsource = stream.getDataSource();
		
		// create a player by passing datasource to the Media Manager
		newplayer = Manager.createPlayer(dsource);
	    }catch (Exception e){
		System.out.println("newRecvStreamEvent exception " +
				   e.getMessage());
		return;
	    }
	    if (newplayer == null)
		return;
	    // send this player to player GUI
	    playerWindow = new PlayerWindow( newplayer );
	}// instanceof newRecvStreamEvent
	
	if (event instanceof PayloadChangeEvent)
	    System.out.println("received a payloadchange event");
	    	    
    }// end of update()


    
    // Since this object is an ActionListener, it must implement the
    // actionPerformed() method in order to receive all AWT events
    public void  actionPerformed(ActionEvent event){
	
	TextField textfield = null;
	Button button = null;
	baddress = false;
	bport = false;
	bmedia = false;
	address = null;
	portstr = null;
	media = null;
	bttl = false;
	ttl = 1;

	//Make the RTPMonitors visible if the buttons are clicked on
	//provided the RTPSessionManagers exist 
	if (event.getSource() instanceof Button){
	    button = (Button)event.getSource();
	    if ((button == videobutton) && (videomgr != null))
		videogui.Show();
	    if ((button == audiobutton) && (audiomgr != null))
		audiogui.Show();
	    return;
	}
	if (event.getSource() instanceof TextField)
	    textfield = (TextField)event.getSource();
	else
	    return;
	// at this point,event is due to one of the text fields
	// process the URL and create the RTPSessionManager if all
	// required feilds of the URL are valid
	String text = textfield.getText();
	text.trim();
	int len = text.length();
	if (len == 0){
	    textfield.setText("URL should be rtp://address/port/ttl/media/");
	    return;
	}
	    
	nm = new char[len];
	text.getChars(6, len, nm, 0);
	
	int i = 0;
	if (i + 7 >= len){
	    textfield.setText("URL should be rtp://address/port/ttl/media/");
	    return;
	}
	while (!baddress){
	    if (i+7 < len){
		char c = nm[i];
		if (c == '/') {
		    address = new String(nm,0,i);
		    baddress = true;
		}
		else
		    i++;
	    }
	    else{
		textfield.setText("URL should be rtp://address/port/ttl/media/");
		return;
	    }
	}
	
	i++;
	offset = i;
	
	if ((i + 6 >= len) || (address == null)){
	    textfield.setText("URL should be rtp://address/port/ttl/media/");
	    return;
	}
	while (!bport){
	    if (i + 6 < len){
		char c = nm[i];
		if (c == '/'){
		    portstr = new String(nm,offset,i- offset);
		    bport = true;
		}
		else
		    i++;
	    }
	    else{
		textfield.setText("URL should be rtp://address/port/ttl/media/");
		return;
	    }
	}
	
	i++;
	offset = i;
	
	if ((i+6 >= len) || (portstr == null)){
	    textfield.setText("URL should be rtp://address/port/ttl/media/");
	    return;
	}
	while(!bttl){
	    if (i + 4 < len){
		char c = nm[i];
		if (c == '/'){
		    ttlstr = new String(nm,offset,i-offset);
		    bttl = true;
		}else
		    i++;
	    }
	    else {
		ttlstr = new String ("1");
		break;
	    }
	}
	i++;
	offset = i;
	
	while (!bmedia){
	    if (i+6 < len){
		char c = nm[i];
		if (c == '/'){
		    media  = new String(nm,offset,i-offset);
		    bmedia = true;
		}
		else
		    i++;
	    }
	    else{
		textfield.setText("URL should be rtp://address/port/ttl/media/");
		return;
	    }
	}
	
	
    
	//the URL has all been validated at this point, now create the
	//RTPSessionManager
    	if ((media.equals("video")) && (videomgr == null)){
	    StartSessionManager(address,
				StrToInt(portstr),
				media,
				StrToInt(ttlstr));
	    if (videomgr == null){
		System.out.println("NUll video manager ");
		return;
	    }
	    // start the RTPMonitoring GUI, it will be made visible by
	    // the Show() method on a user clicking the button
	    videogui = new ParticipantListWindow(videomgr);
	    // the URL textfield cannot be changed
	    urlvideo.setEditable(false);
	    videobutton.setEnabled(true);
	}////if ((media.equals("video") && (videomgr == null) 
	if ((media.equals("audio")) && (audiomgr == null)){
	    StartSessionManager(address,
				StrToInt(portstr),
				media,
				StrToInt(ttlstr));
	    if (audiomgr == null){
		System.out.println("Null Audio Manager");
		return;
	    }
	    audiogui = new ParticipantListWindow(audiomgr); 
	    urlaudio.setEditable(false);
	    audiobutton.setEnabled(true);
	    
	}
    }

    private int StrToInt(String str){
	if (str == null)
	    return -1;
	Integer retint = new Integer(str);
	return  retint.intValue();
    }

   
}// end of class

	

