package Voyager.util;

import java.util.Properties;
import java.io.*;


public class VoyAppContext extends VoyContext
{
    Process vidProc_;
    Process audProc_;
    boolean launchRat_;
    boolean keepMedia_;
    boolean vLaunched_;
    boolean aLaunched_;

  public VoyAppContext( )
  {
    super.init();
    launchRat_ = false;
    keepMedia_ = true;
    vLaunched_ = false;
    aLaunched_ = false;
  }

  public boolean getCLaunched() {
    //chech that procs haven't died
    if (!getALaunched() || !getVLaunched())
      cLaunched_ = false;
    else 
      cLaunched_ = true;

    return cLaunched_;
  }

  private boolean getVLaunched() {
    if (vidProc_ == null)
      vLaunched_ = false;
    if (vLaunched_) {
      try {
	vidProc_.exitValue();
	vLaunched_ = false;
      } catch (IllegalThreadStateException ex) {
	vLaunched_ = true;
      }
    }
    return vLaunched_;
  }
    
  private boolean getALaunched() {
    if (audProc_ == null)
      aLaunched_ = false;
    if (aLaunched_) {
      try {
        audProc_.exitValue();
	aLaunched_ = false;
      } catch (IllegalThreadStateException ex) {
	aLaunched_ = true;
      }
    }
    return aLaunched_;
  }

    public void setRatFlag( boolean qRat )
    {
	launchRat_ = qRat;
    }

    public void setKeepMediaFlag( boolean qKeep )
    {
	keepMedia_ = qKeep;
    }
	
  
  public void showStatus( String str )
  {
    System.out.println( str );
  }

  public void launchMediaClients( String destination )
  {
    String videocommand = "vic "+destination+"/"+getVideoPort();
    
    String audiocommand;
    if( launchRat_ )
	audiocommand = "rat "+destination+"/"+getAudioPort();
    else
	audiocommand = "vat -r "+destination+"/"+getAudioPort();

    launchEm( videocommand , audiocommand );
  }

  public void launchMediaClients( String audioDest , String videoDest ,
				  String audioPort , String videoPort )
  {
    String videocommand = "vic "+videoDest+"/"+videoPort;
    
    String audiocommand;
    if( launchRat_ )
	audiocommand = "rat "+audioDest+"/"+audioPort;
    else
	audiocommand = "vat -r "+audioDest+"/"+audioPort;

    launchEm( videocommand , audiocommand );
  }
    private void launchEm( String videoCmd , String audioCmd )
    {
    try
      {
	Runtime runTime = Runtime.getRuntime();
	if (!getVLaunched()) vidProc_ = runTime.exec(videoCmd);
	vLaunched_ = true;
	if (!getALaunched()) audProc_ = runTime.exec(audioCmd);
	aLaunched_ = true;

	cLaunched_ = true;
      }
    catch( IOException e )
      {
	System.err.println( e );
      }
    finally
      {
	System.out.println( "VoyAppContext::launchMediaClients: video " + videoCmd );
	System.out.println( "VoyAppContext::launchMediaClients:audio " + audioCmd );
      }
    }

    public void killMediaClients( )
    {
	if( !keepMedia_ )
	{
	    if( vidProc_ != null )
	    {
		System.out.println( "Destroying vidProc " + vidProc_ );
		vidProc_.destroy( );
		vidProc_ = null;
	    }

	    if( audProc_ != null )
	    {
		System.out.println( "Destroying audProc " + audProc_ );
		audProc_.destroy( );
		audProc_ = null;
	    }
	    
	    cLaunched_ = false;
	    vLaunched_ = false;
	    aLaunched_ = false;
	}
    }
	

  public org.omg.CORBA.ORB initCorba(  )
  {
    String args[ ] = new String[0];

    //showStatus("Getting orb");
    orb_ = org.omg.CORBA.ORB.init( args , new java.util.Properties( ) );
    
    //showStatus("Returning orb");
    return orb_;
  }

  public void shutdown() {
    keepMedia_ = false;
    if (cLaunched_) killMediaClients();
    java.lang.Runtime rt = java.lang.Runtime.getRuntime();
    rt.exit(0);
  }
}


    
