#include "Corba_Manager.h"

#if HAVE_LIBOB
#include <OB/Util.h>
#endif

#if HAVE_LIBACE
#include <ace/SOCK_Stream.h>
#include <ace/SOCK_Connector.h>
#include <ace/IOStream.h>

typedef ACE_IOStream<ACE_SOCK_Stream> ACE_SOCK_IOStream;
#endif

#include <stdio.h>
#include <fstream.h>

Corba_Manager::Corba_Manager()
{
}

int Corba_Manager::open(int &argc, char **argv)
{
    int rc = 0;
    try {
	orb_ = CORBA_ORB_init(argc, argv);
	boa_ = orb_ -> BOA_init(argc, argv);
	
#ifdef HAVE_LIBOB
	//	orb_->nested(true);
#endif
    }
#ifdef __GNUG__
    catch(CORBA_COMM_FAILURE& ex)
#else
    catch(CORBA_SystemException& ex)
#endif
    {
#ifdef HAVE_LIBOB
	printf("OB Initialize failed\n");
	OBPrintException(ex);
#endif
	
	rc = 1;
    }

    return rc;
}

#if HAVE_LIBACE

CORBA_Object_ptr Corba_Manager::objectFromWeb(VOY_String host, VOY_String path,
					      int port)
{
    ACE_SOCK_IOStream s;
    
    ACE_SOCK_Connector c;

    if (c.connect(s, ACE_INET_Addr(port, host.c_str())) == -1)
    {
	printf("Cannot connect to web server at %s/%d\n", host.c_str( ), port);
	return CORBA_Object::_nil();
    }

    s << "GET " << path.c_str( ) << endl;
    s << endl;

    /*
     * Someday we'll figure out how to do real HTTP 1.0.
    char line[1024];

    while (1)
    {
	s.getline(line, sizeof(line));
	printf("line: %s\n", line);
	if (line[0] == '\0')
	    break;
    }
     */

    char strbuf[1000];
    s >> strbuf;
    printf("Read buf: '%s'\n", strbuf);

    s.close();

    CORBA_Object_ptr obj;

    obj = orb_ -> string_to_object(strbuf);
    
    return obj;
}

#endif // HAVE_LIBACE

int Corba_Manager::objectToIORFile(CORBA_Object_ptr obj, VOY_String file)
{
    CORBA_String_var s = orb_ -> object_to_string(obj);

    ofstream out(file.c_str());
    if(out.fail())
    {
	fprintf(stderr, "writeObjref: cannot write %s\n", file.c_str( ));
	return 1;
    }

    //
    // VC++ didn't find the CORBAString_var operator at linktime..
    //
    out << (char *) s << endl;
    out.close();
    return 0;
}

CORBA_Object_ptr Corba_Manager::objectFromIORFile(VOY_String file)
{
    ifstream in(file.c_str());
    if (in.fail())
    {
	cout << "Cannot read IOR file " << file.c_str( ) << endl;
	return CORBA_Object::_nil();    
    }
    char s[1000];
    in >> s;

    CORBA_Object_ptr obj = orb_ -> string_to_object(s);
    return obj;
}
