#ifndef _Network_Transport_h_
#define _Network_Transport_h_

#include "config.h"
#include <pair.h>

/////////////////////////////////////////////////////////////////////////////
//
// Network_Transport.h
//
// Revision History
//     28 Sep 97 - lisa - original version
//

class RTP_Packet;

#include <ace/SOCK_Dgram.h>
#include <ace/SOCK_Dgram_Mcast.h>
#include <ace/INET_Addr.h>

#include <ace/Event_Handler.h>

class Network_Transport : public ACE_Event_Handler
{
public:
    class EndpointIdentifier
    {
    public:
	EndpointIdentifier() :
	    host(""), port(-1), ttl(-1), any(0)
	{}

 	EndpointIdentifier(const VOY_String &h, int p, int t = -1) :
	    host(h), port(p), ttl(t), any(0)
	{}

 	EndpointIdentifier(int p) :
	    host(""), port(p), ttl(-1), any(1)
	{}

 	EndpointIdentifier(const EndpointIdentifier &ei) :
	    host(ei.host), port(ei.port), ttl(ei.ttl), any(ei.any)
	{}

 	void set(const VOY_String &h, int p, int t = -1)
	{
	    host = h;
	    port = p;
	    ttl = t;
	    any = 0;
	}

 	void set(int p)
	{
	    host = "";
	    port = p;
	    ttl = -1;
	    any = 1;
	}

	bool operator==(const EndpointIdentifier &ei) const
	{
	    return ei.host == host && ei.port == port && ei.ttl == ttl &&
		ei.any == any;
	}

	bool operator<(const EndpointIdentifier &ei) const
	{
	    if (host < ei.host)
		return true;
	    else if (host > ei.host)
		return false;

	    if (port < ei.port)
		return true;
	    else if (port > ei.port)
		return false;

	    if (ttl < ei.ttl)
		return true;
	    else if (ttl > ei.ttl)
		return false;

	    return ei.any != any;
	}

	bool isMulticast()
	{
	    if (any)
		return 0;
	    ACE_INET_Addr addr(port, host.c_str());

	    return IN_CLASSD(addr.get_ip_address());
	}

	ACE_INET_Addr rtpAddress()
	{
	    if (any)
		return ACE_INET_Addr(port);
	    else
		return ACE_INET_Addr(port, host.c_str());
	}

	ACE_INET_Addr rtcpAddress()
	{
	    if (any)
		return ACE_INET_Addr(port + 1);
	    else
		return ACE_INET_Addr(port + 1, host.c_str());
	}

	VOY_String host;
	int port;
	int ttl;
	int any;
    };

    virtual Network_Transport(EndpointIdentifier endpoint) = 0;
    virtual ~Network_Transport( );

    int isMulticast( void ) { return isMulticast_; }
    const ACE_SOCK_Dgram_Mcast& rtpMcastSock( void ) { return rtp_mc_sock_; }
    const ACE_SOCK_Dgram_Mcast& rtcpMcastSock( void ) { return rtcp_mc_sock_; }
    const ACE_SOCK_Dgram& rtpSock( void ) { return rtp_sock_; }
    const ACE_SOCK_Dgram& rtcpSock( void ) { return rtcp_sock_; }

    const EndpointIdentifier& endpoint() const { return endpoint_; }

private:
    EndpointIdentifier endpoint_;

    ACE_INET_Addr rtp_address_;
    ACE_INET_Addr rtcp_address_;

    int isMulticast_;
    ACE_SOCK_Dgram rtp_sock_;
    ACE_SOCK_Dgram rtcp_sock_;

    ACE_SOCK_Dgram_Mcast rtp_mc_sock_;
    ACE_SOCK_Dgram_Mcast rtcp_mc_sock_;
};

inline ostream &operator<<(ostream &os,
			   const Network_Transport::EndpointIdentifier &ei)
{
    if (ei.any)
    {
	os << "*/" << ei.port;
    }
    else
    {
	os << ei.host << "/" << ei.port << "/" << ei.ttl;
    }
    return os;
}

inline ostream &operator<<(ostream &os, const Network_Transport &nt)
{
    const Network_Transport::EndpointIdentifier &ei = nt.endpoint();
    os << "Network transport: endpoint=" << ei;
    os << endl;
    return os;
}
    
#endif
