#include "Play_Session_Manager.h"
#include "RTSP_Setup_Request.h"
#include "RTSP_Play_Request.h"
#include "Play_Stream_Synchronizer.h"
#include "Play_Stream_Manager.h"
#include "Playback_Network_Transport.h"
#include "RTSP_Response.h"
#include "RTSP_Request_Packet.h"
#include "RTSP_Teardown_Request.h"
#include "ace/OS.h"


/////////////////////////////////////////////////////////////////////////////
//
// Play_Session_Manager.C
//
// Revision History
//      12 Sep 97 - lisa - original version
//


Play_Session_Manager::Play_Session_Manager( VOY_Database_ptr db ,
					    Play_Session_ID id ,
					    VOY_String payFreqFileName ) :
    databaseRef_( db ) , sessionId_( id ) , payFreqFileName_( payFreqFileName )
{
    synch_  = new Play_Stream_Synchronizer( sessionId_ );

    if( ! synch_  )    
    {
	assert( 0 );
	// throw exception
    }
}

Play_Session_Manager::~Play_Session_Manager( )
{
    StreamMap::iterator  iter;
    for( iter = streamMap_.begin( ) ; iter != streamMap_.end(  ) ; iter++ )
    {
	delete (*iter).second;
	streamMap_.erase( iter );
    }
}

Play_Session_Manager::PlaySessionError
Play_Session_Manager::addStream( RTSP_Setup_Request* sRequest ,
				 Playback_Network_Transport* netTrans )
{
    PlaySessionError errCode = kFailure;

    assert( synch_   &&  sRequest  &&  netTrans );
    
    Play_Stream_Manager* playStream = new Play_Stream_Manager( databaseRef_ ,
							  sRequest  , synch_ ,
							  netTrans ,
							  payFreqFileName_ );
    printf( "Play_Session_Manager::addStream: after creating Play_Stream_Manager %u\n"  ,
	    playStream );
    if( playStream )
    {
	char streamIdc[ 256 ];
	sRequest->streamId( streamIdc );
	VOY_String streamId( streamIdc );
	streamMap_[ streamId ] = playStream;

	printf( "Play_Session_Manager::addStream: assigned stream %s to Play_Stream_Manager %u, mapsize=%d\n" , streamId.c_str( ) , playStream , streamMap_.size( ) );

	printf( "Play_Session_Manager::addStream: sending response to setup, id=%d\n",
		(int) sessionId_ );
	RTSP_Request_Packet *request = sRequest->requestPacket( );
	RTSP_Response resp( 200 , "OK" , atoi( request->sequence( ) ) );
	char sessionId[ 256 ];
	sprintf( sessionId , "%d" , (int) sessionId_ );
	resp.setHeader( "Session" , sessionId );
	resp.send( request->engine( ) );

    	errCode = kSuccess;
}

    return errCode;
}

Play_Session_Manager::PlaySessionError
Play_Session_Manager::play( RTSP_Play_Request *request )
{
    PlaySessionError errCode = kSuccess;

    StreamMap::iterator iter;
    for( iter = streamMap_.begin( ) ; iter != streamMap_.end(  ) ; iter++ )
    {
	// As of 10.22.97, Play_Stream_Manager::play( ) is a noop
	printf( "Play_Session_Manager::play: calling play( ) for %u\n" , (*iter).second );
	(*iter).second->play( request );
    }

    synch_->play( request );

    return errCode;
}

void
Play_Session_Manager::teardown( RTSP_Teardown_Request* tRequest ,
			                short& qNoMoreStreams )
{
    char streamIdc[ 256 ];
    tRequest->streamId( streamIdc );
    VOY_String streamId( streamIdc );

    printf( "Play_Session_Manager::teardown: tearing down stream %s\n" , streamId.c_str( ) );

    StreamMap::iterator pStreamMan = streamMap_.find( streamId );

    if( pStreamMan != streamMap_.end( ) )
    {
 	printf( "Play_Session_Manager::teardown: GOOD TEARDOWN!\n" );
	
	// Pass the teardown request on to its Play_Stream_Manager
	(*pStreamMan).second->teardown( );

	// Clean up time
	//
	delete (*pStreamMan).second;
	streamMap_.erase( pStreamMan );
	
	// Send a response to the teardown request
	RTSP_Request_Packet *request = tRequest->requestPacket( );
	RTSP_Response resp( 200 , "OK" , atoi( request->sequence( ) ) );
	char sessionId[ 256 ];
	sprintf( sessionId , "%d" , (int) sessionId_ );
	resp.setHeader( "Session" , sessionId );
	resp.send( request->engine( ) );
    }
    else
    {
	// Send a bad response
	printf( "Play_Session_Manager::teardown: BAD TEARDOWN!\n" );
	assert( 0 );
    }
    
// If there are no more streams, the parent Playback_Manager will delete
// this Play_Session_Manager
qNoMoreStreams = streamMap_.empty( );
}
