#include "config.h"
#include "Play_Stream_Synchronizer.h"
#include "Stream_Source.h"
#include "S_Time.h"
#include "N_Time.h"
#include "RTSP_Play_Request.h"
#include "RTSP_Response.h"
#include "RTSP_Request_Packet.h"


/////////////////////////////////////////////////////////////////////////////
//
// Play_Stream_Synchronizer.C
//
// Revision History
//     17 Sep 97 - lisa - original version
//

Play_Stream_Synchronizer::Play_Stream_Synchronizer( Play_Session_ID id ) : minTimeNotSet_( 1 ) , sessionId_( id )
{
}

Play_Stream_Synchronizer::~Play_Stream_Synchronizer( )
{
    printf( "Play_Stream_Synchronizer::~Play_Stream_Synchronizer: !!\n" );
}

void Play_Stream_Synchronizer::registerStream( Stream_Source* src ,
					       ACE_Time_Value time )
{
    printf( "Play_Stream_Synchronizer::registerStream: Stream_Source %u is registering\n" ,  src );
    assert( src );

    if( minTimeNotSet_ )
    {
	minTime_ = time;
	minTimeNotSet_ = 0;
    }
    else if( time < minTime_  )
	minTime_ = time;

    sourceList_.insert( sourceList_.end( ) ,  src );
}

void Play_Stream_Synchronizer::unregisterStream( Stream_Source* src )
{
    //printf( "Play_Stream_Synchronizer::unregisterStream: a Stream_Source is dropping it's registration\n" );

    sourceList_.remove( src );

    // If we don't have anymore active streams, our work is done
    if( sourceList_.empty( ) )  
	delete this;
}

void Play_Stream_Synchronizer::play( RTSP_Play_Request* pRequest )
{
    //printf( "Play_Stream_Synchronizer::play:  Got a play request\n" );
    assert( pRequest );

    ACE_Time_Value playbackStart = ACE_OS::gettimeofday();

    list< Stream_Source* >::iterator iter;
   
    for( iter = sourceList_.begin( ) ; iter != sourceList_.end( ) ; iter++ )
	(*iter)->play( pRequest , playbackStart , minTime_  );

    // Send a response to the play request
    RTSP_Request_Packet *request = pRequest->requestPacket( );
    RTSP_Response resp( 200 , "OK" , atoi( request->sequence( ) ) );
    char sessionId[ 256 ];
    sprintf( sessionId , "%d" , (int) sessionId_ );
    resp.setHeader( "Session" , sessionId );
    resp.send( request->engine( ) );
}

