#include "Playback_Network_Transport.h"
#include "RTP_Packet.h"

/////////////////////////////////////////////////////////////////////////////
//
// Playback_Network_Transport.C
//
// Revision History
//      26 jan 98 - lisa - adapted from Network_Transport
//

#include <sys/errno.h>
#include <stream.h>
#include <ace/Reactor.h>

Playback_Network_Transport::Playback_Network_Transport( Network_Endpoint ePoint ) :
    endpoint_( ePoint ) , rtp_address_( ePoint.rtpAddress( ) ) , qError_( 0 ) ,
    rtcp_address_( ePoint.rtcpAddress( ) ) , isMulticast_( ePoint.isMulticast( ) )
{
    //cout << form( "Playback_Network_Transport::Playback_Network_Transport: %x for %s/%d\n", this , rtp_address_.get_host_addr( ) , rtp_address_.get_port_number( ) );
    //cout << "Playback_Network_Transport::Playback_Network_Transport: endpoint is " << ePoint << endl;

    if( isMulticast_ )
    {
	int rc;
	rc = rtp_mc_sock_.subscribe( rtp_address_ );
	if( rc < 0 )
	{
	    perror( "error on rtp subscribe" );
	    qError_ = 1;
	}
	    
	rc = rtcp_mc_sock_.subscribe( rtcp_address_ );
	if( rc < 0 )
	{
	    perror( "error on rtcp subscribe" );
	    qError_ = 1;
	}
	    
	//ePoint.ttl_ = 16;
	if( ePoint.ttl_ > 1 )
	{
	  //	    int h;
	  //	    u_char ttl;

	    if( rtp_mc_sock_.set_option( IP_MULTICAST_TTL , ePoint.ttl_  ) < 0 )
	    {
		perror( "error on rtp set ttl" );
		qError_ = 1;
	    }
	    if( rtcp_mc_sock_.set_option( IP_MULTICAST_TTL , ePoint.ttl_ ) < 0 )
	    {
		perror( "error on rtcp set ttl" );
		qError_ = 1;
	    }
	    if( rtp_mc_sock_.set_option( IP_MULTICAST_LOOP, 0 ) < 0 )
	    {
		perror( "error on rtp set loop" );
		qError_ = 1;
	    }
	    if( rtcp_mc_sock_.set_option (IP_MULTICAST_LOOP, 0 ) < 0 )
	    {
		perror( "error on rtcp set loop" );
		qError_ = 1;
	    }
	}

	//
	// Register for updates
	//
	ACE_Reactor::instance( )->register_handler (rtp_mc_sock_.get_handle( ) , this ,
						ACE_Event_Handler::READ_MASK);
	ACE_Reactor::instance( )->register_handler( rtcp_mc_sock_.get_handle( ) , this ,
						ACE_Event_Handler::READ_MASK);
    }
    else
    {
	if( rtp_sock_.open( ACE_INET_Addr( ePoint.port_  ) ) < 0 )
	{
	    perror( "Playback_Network_Transport::Playback_Network_Transport: rtp_sock open" );
	    qError_ = 1;
	}

	if( rtcp_sock_.open( ACE_INET_Addr(ePoint.port_ + 1 ) ) < 0 )
	{
	    perror( "Playback_Network_Transport::Playback_Network_Transport: rtcp_sock open" );
	    qError_ = 1;
	}

	ACE_Reactor::instance( )->register_handler( rtp_sock_.get_handle( ) , this ,
						ACE_Event_Handler::READ_MASK);
	ACE_Reactor::instance( )->register_handler(rtcp_sock_.get_handle( ) , this ,
						ACE_Event_Handler::READ_MASK);
    }
}

Playback_Network_Transport::~Playback_Network_Transport( )
{
}

void
Playback_Network_Transport::playRtpPacket( int type , char* packet , int len )
{
    //printf( "Playback_Network_Transport::playRtpPacket: playing RTP\n" );

    int rc;
    if( isMulticast_ )
	rc = rtp_mc_sock_.send( (const void* )packet , len );
    else
	rc = rtp_sock_.send( (const void* )packet , len , rtp_address_ );

    if( rc < 0 )
    {
	fprintf( stderr , "Playback_Network_Transport::playRtpPacket: send %d failed: %s\n" ,
		len , strerror( errno ) );
    }

}

void
Playback_Network_Transport::playRtcpPacket( char* packet , int len )
{
    //printf( "Playback_Network_Transport::playRtcpPacket: playing RTCP\n" );
    int rc;

    if( isMulticast_ )
	rc = rtcp_mc_sock_.send( ( const void* )packet , len );
    else
	rc = rtcp_sock_.send( ( const void* )packet , len , rtcp_address_ );

    if( rc < 0 )
    {
	cout << form( "Playback_Network_Transport::playRtcpPacket: send %d failed: %s\n" ,
		len , strerror( errno ) );
	cout << "Playback_Network_Transport::playRtcpPacket: transport is " << *this << endl;
    }
}

int Playback_Network_Transport::handle_input( ACE_HANDLE fd )
{
    if( fd == rtp_sock_.get_handle( ) )
    {
	printf("Playback_Network_Transport::handle_input: input on rtp_sock.  host = %s, port = %d\n" , rtp_address_.get_host_addr( ) , rtp_address_.get_port_number( ));
    }
    else if( fd == rtcp_sock_.get_handle( ) )
    {
	printf("Playback_Network_Transport::handle_input: input on rtcp_sock.  host = %s, port = %d\n" , rtcp_address_.get_host_addr( ) , rtcp_address_.get_port_number( ));
    }
    else if( fd == rtp_mc_sock_.get_handle( ) )
    {
	printf("Playback_Network_Transport::handle_input: input on rtp_mc_sock.  host = %s, port = %d\n" , rtp_address_.get_host_addr( ) , rtp_address_.get_port_number( ));
    }
    else if( fd == rtcp_mc_sock_.get_handle( ) )
    {
	printf("Playback_Network_Transport::handle_input: input on rtcp_mc_sock.  host = %s, port = %d\n" , rtcp_address_.get_host_addr( ) , rtcp_address_.get_port_number( ));
    }
    char buf[ 4096 ];
    int n = ACE_OS::recv( fd , buf , sizeof( buf ) );
    //printf ("Playback_Network_Transport::handle_input: read %d bytes\n" , n );
    return 0;
}
