#include "RTSP_Engine.h"
#include "RTSP_Response.h"

#include "Dispatcher.h"

#include <stdarg.h>

#define MAX_HEADER 8192

RTSP_Engine::RTSP_Engine(Dispatcher *d) :
    mb_(MAX_HEADER),
    dispatcher_(d),
    request_(NULL)
{
//    printf("Engine create sm=%x\n", sm);
}

RTSP_Engine::RTSP_Engine() :
    mb_(MAX_HEADER),
    dispatcher_(NULL),
    request_(NULL)
{
//    printf("Engine create -- why me?\n");
}

int RTSP_Engine::open(void *)
{
    ACE_Service_Config::reactor()->register_handler(this, ACE_Event_Handler::READ_MASK);

    peer().get_remote_addr(peerAddr_);
    
    return 0;
}

/**
  Handle input from the client.
  */

int RTSP_Engine::handle_input(ACE_HANDLE fd)
{
//    printf("Handle input!\n");

    int result = peer().recv(mb_.wr_ptr(), MAX_HEADER - mb_.length());

//    printf("recv returns %d\n", result);

    if (result == 0)
    {
//	printf("Got eof for engine\n");
	ACE_Service_Config::reactor ()->remove_handler
	    (this,  ACE_Event_Handler::READ_MASK);
	return -1;
    }

    if (request_ == NULL)
    {
//	printf("New request\n");
	request_ = new RTSP_Request_Packet(this);
    }

    mb_.wr_ptr(result);
    int parse_status = request_->parse_request(mb_);
//    printf("parse_status=%d\n", parse_status);

    if (parse_status == 1)
    {
//	printf("request done!\n");
	handleRequest(request_);
	request_ = NULL;
    }

    return 0;
}

void RTSP_Engine::handleRequest(RTSP_Request_Packet *request)
{
    try {
	dispatcher_->handleRequestPacket(request);
    }
    catch (RTSP_Response *resp)
    {
	printf("Threw an exception: %d %s\n", resp->code(),
	       resp->reason());
	resp->send(request->engine());
    }
}

void RTSP_Engine::sendf(char *fmt, ...)
{
    va_list argp;
    char buf[8192];

    va_start(argp, fmt);
    ACE_OS::vsprintf(buf, fmt, argp);
    va_end(argp);

    peer().send(buf, strlen(buf));
}
