#include "RTSP_Request.h"
#include "RTSP_Request_Packet.h"
#include "ace/OS.h"
#include <string.h>

#include "RTSP_Describe_Request.h"
#include "RTSP_Setup_Request.h"
#include "RTSP_Play_Request.h"
#include "RTSP_Teardown_Request.h"
#include "RTSP_Record_Request.h"

//////////////////////////////
//
// Virtual constructor for creating requests from request packets

RTSP_Request *RTSP_Request::createRequest(RTSP_Request_Packet *packet)
{
    switch (packet->type())
    {
    case RTSP_Request_Packet::DESCRIBE:
    {
	RTSP_Describe_Request *describeRequest = new RTSP_Describe_Request();
	if (describeRequest->setRequest(packet) < 0)
	{
	    cout << "describe request failed!\n";
	    return 0;
	}
	else
	{
	    cout << "COUT: " << *describeRequest << endl;
	    return describeRequest;
	}
    }

    case RTSP_Request_Packet::SETUP:
    {
	RTSP_Setup_Request *setupRequest = new RTSP_Setup_Request();
	if (setupRequest->setRequest(packet) < 0)
	{
	    cout << "setup request failed!\n";
	    return 0;
	}
	else
	{
	    return setupRequest;
	}
    }
    case RTSP_Request_Packet::PLAY:
    {
	RTSP_Play_Request *playRequest = new RTSP_Play_Request();
	if (playRequest->setRequest(packet) < 0)
	{
	    cout << "play request failed!\n";
	    return 0;
	}
	else
	{
	    return playRequest;
	}
    }
    case RTSP_Request_Packet::TEARDOWN:
    {
	RTSP_Teardown_Request *tearRequest = new RTSP_Teardown_Request;
	
	if( tearRequest && tearRequest->setRequest( packet ) < 0 )
 	    cout << "RTSP_Request::createRequest:  teardown request failed!\n";
	
	return tearRequest;
    }
    case RTSP_Request_Packet::RECORD:
    {
	RTSP_Record_Request* recordRequest = new RTSP_Record_Request( );
	if( recordRequest->setRequest( packet ) < 0 )
	{
	    cout << "record request failed!\n";
	    return 0;
	}
	else
	    return recordRequest;
    }
    }
    
    return 0;
}

//////////////////////////////////////////
//
// Real constructor.

RTSP_Request::RTSP_Request() :
    requestPacket_(0), streamId_(NULL)
{
}

RTSP_Request::~RTSP_Request( )
{
    if( streamId_ )
	ACE_OS::free( streamId_ );
    streamId_ = NULL;
}

int RTSP_Request::setRequest( RTSP_Request_Packet *r )
{
    char *uri = ACE_OS::strdup( r->path() );
    
    sessionId_ = atoi( uri );
    
    char *slash = strchr( uri , '/' );
    streamId_ = slash ? strdup( slash+1) : "";
    
    ACE_OS::free( uri );

    return setRequestPacket( r );
}

void RTSP_Request::streamId(char *streamId) const
{
    strcpy( streamId, streamId_ );
}

ostream& RTSP_Request::put(ostream &os) const
{
    return os << "RTSP_Request()";
}

