
#include "RTSP_Response.h"
#include "RTSP_Engine.h"

#include <stdarg.h>

RTSP_Response::RTSP_Response(int code, char *reason, int sequence) :
    code_(code), reason_(reason), body_(0), sequence_(sequence), version_("RTSP/1.0")
{
}

RTSP_Response::RTSP_Response(int code, char *reason, const char *sequence) :
    code_(code), reason_(reason), body_(0), sequence_(atoi(sequence)), version_("RTSP/1.0")
{
}

void RTSP_Response::setHeader(char *hdr, char *value)
{
    printf("setHeader %s %s\n", hdr, value);
    headerMap_[hdr] = value;
}

void RTSP_Response::bodyInit(int size)
{
    body_ = new char[size];
    *body_ = '\0';
}

void RTSP_Response::bodyAppend(const char *s)
{
//    printf("Append '%s'\n", s);
    ACE_OS::strcat(body_, s);
}

void RTSP_Response::bodyAppendf(const char *fmt, ...)
{
    va_list argp;
    char buf[8192];

    va_start(argp, fmt);
    ACE_OS::vsprintf(buf, fmt, argp);
    va_end(argp);
    bodyAppend(buf);
}

void RTSP_Response::bodyDone()
{
    int len = ACE_OS::strlen(body_);
    char b[20];
    sprintf(b, "%d", len);
    setHeader("Content-Length", b);
}

void RTSP_Response::send(RTSP_Engine *engine)
{
    engine->sendf("%s %d %d %s\n", version(), code(), sequence(), reason());

    Header_Map::iterator mapIter;

    for (mapIter = headerMap_.begin(); mapIter != headerMap_.end();
	 mapIter++)
    {
	const char *key = (*mapIter).first.c_str();
	const char *value = (*mapIter).second.c_str();
	printf( "RTSP_Response::send: Have entry %s %s\n", key, value);
	engine->sendf("%s: %s\n", key, value);
    }
    engine->sendf("\n");
    if (body_)
    {
	engine->peer().send(body_, strlen(body_));
    }
}
