#include "RTSP_Transport_Info.h"

#include <ace/OS.h>
#include <ctype.h>

RTSP_Transport_Info::RTSP_Transport_Info() :
    valid_(0), append_(0), mode_(kPlay)
{
}

//
// Format is
//    transport-protocol '/' profile [ '/' lowerTransport ]
//		';' parameter ...
//
int RTSP_Transport_Info::parse(const char *header)
{
    header_ = headerCur_ = ACE_OS::strdup(header);

    int rc = s_init();

    cout << "parse returns " << rc << endl;

    ACE_OS::free(header_);

    return rc;
}

RTSP_Transport_Info::Token &RTSP_Transport_Info::getToken()
{
    switch (*headerCur_)
    {
    case '\0':
	currentToken_.type_ = Token::kEOF;
	headerCur_++;
	break;

    case '/':
	currentToken_.type_ = Token::kSlash;
	headerCur_++;
	break;
	
    case ';':
	currentToken_.type_ = Token::kSemicolon;
	headerCur_++;
	break;

    case '"':
	currentToken_.type_ = Token::kDoubleQuote;
	headerCur_++;
	break;

    case '=':
	currentToken_.type_ = Token::kEquals;
	headerCur_++;
	break;

    default:
	if (isalnum(*headerCur_))
	{
	    char *t = headerCur_;
	    while (isalnum(*headerCur_) ||
		   *headerCur_ == '.' ||
		   *headerCur_ == '-')
	    {
		headerCur_++;
	    }
	    VOY_String tok(t, headerCur_ - t);
	    currentToken_.string_ = tok;
	    currentToken_.type_ = Token::kString;
	}
	else
	    currentToken_.type_ = Token::kInvalid;
	break;
    }
//    cout << "got token, type=" << currentToken_.type() << endl;
//    if (currentToken_.type() == Token::kString)
//	cout << "  string=" << currentToken_.getString() << endl;
    return currentToken_;
}

int RTSP_Transport_Info::s_init()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kString:
	transportProtocol_ = token.getString();
	return s_1();
    }

    return -1;
}

int RTSP_Transport_Info::s_1()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kSlash:
	return s_2();
    }
    
    return -1;
}

int RTSP_Transport_Info::s_2()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kString:
	profile_ = token.getString();
	return s_3();

    case Token::kEOF:
	return 0;
    }

    return -1;
}

int RTSP_Transport_Info::s_3()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kSlash:
	return s_4();

    case Token::kSemicolon:
	return s_6();
    }

    return -1;
}

int RTSP_Transport_Info::s_4()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kString:
	lowerTransport_ = token.getString();
	return s_5();
    }

    return -1;
}

int RTSP_Transport_Info::s_5()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kSemicolon:
	return s_6();

    case Token::kEOF:
	return 0;
    }

    return -1;
}

int RTSP_Transport_Info::s_6()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kString:
	paramName_ = token.getString();

	cout << "s6: check " << paramName_ << endl;
	// This is a stlport bug
	if (paramName_ == VOY_String("mode"))
	{
	    cout << "got mode\n";
	    return s_10();
	}
	else
	    return s_7();
    }

    return -1;
}

int RTSP_Transport_Info::s_7()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kEquals:
	return s_8();

    case Token::kSemicolon:
	setParameter(paramName_, "");
	paramName_ = "";
	return s_6();

    case Token::kEOF:
	return 0;
    }

    return -1;
}

int RTSP_Transport_Info::s_8()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kString:
	setParameter(paramName_, token.getString());
	return s_9();
    }

    return -1;
}

int RTSP_Transport_Info::s_9()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kSemicolon:
	return s_6();

    case Token::kEOF:
	return 0;
    }

    return -1;
}

int RTSP_Transport_Info::s_10()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kEquals:
	return s_11();
    }

    return -1;
}

int RTSP_Transport_Info::s_11()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kDoubleQuote:
	return s_12();
    }

    return -1;
}

int RTSP_Transport_Info::s_12()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kString:
	if (token.getString() == VOY_String("RECORD"))
	{
	    mode_ = kRecord;
	    cout << "got record\n";
	    return s_13();
	}
	else if (token.getString() == VOY_String("PLAY"))
	{
	    mode_ = kPlay;
	    cout << "got play\n";
	    return s_18();
	}
	break;
    }

    return -1;
}

int RTSP_Transport_Info::s_13()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kSemicolon:
	return s_14();

    case Token::kDoubleQuote:
	return s_19();
    }

    return -1;
}

int RTSP_Transport_Info::s_14()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kString:
	cout << "got record param " << token.getString() << endl;
	if (token.getString() == VOY_String("append"))
	    append_ = 1;

	return s_15();

    case Token::kDoubleQuote:
	return s_19();
    }

    return -1;
}

int RTSP_Transport_Info::s_15()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kEquals:
	return s_16();

    case Token::kDoubleQuote:
	return s_19();
    }

    return -1;
}

int RTSP_Transport_Info::s_16()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kString:
	cout << "got record param value " << token.getString() << endl;

	return s_17();
    }

    return -1;
}

int RTSP_Transport_Info::s_17()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kSemicolon:
	return s_14();
    }

    return -1;
}

int RTSP_Transport_Info::s_18()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kDoubleQuote:
	return s_19();
    }

    return -1;
}


int RTSP_Transport_Info::s_19()
{
    Token &token = getToken();

    switch (token.type())
    {
    case Token::kSemicolon:
	return s_6();

    case Token::kEOF:
	return 0;
    }

    return -1;
}

void RTSP_Transport_Info::setParameter(const VOY_String &param, const VOY_String &value)
{
    cout << "binding " << param.c_str( ) << " to " << value << endl;
    parameters_[param] = value;
}

ostream& operator<<(ostream &os, RTSP_Transport_Info &r)
{
    os << "transportProtocol: " << r.transportProtocol_ << endl;
    os << "profile: " << r.profile_ << endl;
    os << "lowerTransport: " << r.lowerTransport_ << endl;
    os << "mode: " << (r.mode_ == RTSP_Transport_Info::kPlay ? "PLAY" : "RECORD") << endl;
    os << "append: " << r.append_ << endl;
//    os << "parameters:\n";
//    os << r.parameters_;

    return os;
}
	

