#ifndef _RTSP_Transport_Info_h
#define _RTSP_Transport_Info_h

#include "config.h"
#include <map.h>

#include <stream.h>

class RTSP_Transport_Info
{
public:
    RTSP_Transport_Info();
    int parse(const char *header);

    const VOY_String &transportProtocol() const
    	{ return transportProtocol_; }

    const VOY_String &profile() const { return profile_; }

    const VOY_String &lowerTransport() const
    	{ return lowerTransport_; }

    typedef map<VOY_String, VOY_String, less<VOY_String> > Parameter_Map;

    const Parameter_Map &parameters() const { return parameters_; }

    int getParameter(const VOY_String &param,
		     VOY_String &value) {
	Parameter_Map::iterator iter;
	iter = parameters_.find(param);
	if (iter == parameters_.end())
	    return -1;
	value = (*iter).second;
	return 0;
    }

    friend ostream& operator<<(ostream &os, RTSP_Transport_Info &r);

    enum ModeType
    {
	kRecord,
	kPlay
    };

    ModeType mode() { return mode_; }
    int isPlay() { return mode_ == kPlay; }
    int isRecord() { return mode_ == kRecord; }

protected:
    int valid_;
    
    VOY_String transportProtocol_;
    VOY_String profile_;
    VOY_String lowerTransport_;
    
    Parameter_Map parameters_;

    ModeType mode_;
    int append_;
    
    void setParameter(const VOY_String & name, const VOY_String & value);

    // State machine

    int s_init();
    int s_1();
    int s_2();
    int s_3();
    int s_4();
    int s_5();
    int s_6();
    int s_7();
    int s_8();
    int s_9();
    int s_10();
    int s_11();
    int s_12();
    int s_13();
    int s_14();
    int s_15();
    int s_16();
    int s_17();
    int s_18();
    int s_19();

    struct Token
    {
	VOY_String string_;

	enum Type {
	    kInvalid,
	    kString,
	    kSlash,
	    kSemicolon,
	    kEquals,
	    kDoubleQuote,
	    kEOF
	};
	Type type_;
	Type type() { return type_; }
	const VOY_String &getString() { return string_; }
	
    };

    Token &getToken();
    Token currentToken_;

    VOY_String paramName_;

    char *header_;
    char *headerCur_;
};


#endif /* _RTSP_Transport_Info_h */
