#include "Record_Archive_Manager.h"
#include "Stream_Archive.h"
#include "RTSP_Record_Request.h"
#include "RTSP_Response.h"
#include "RTSP_Request_Packet.h"
#include <ace/OS.h>

Record_Archive_Manager::Record_Archive_Manager( VOY_String directory , int dbSessionId ,
						Record_Session_ID id ,
						VOY_Database_ptr db ) :
    recordFlag_( 0 ) , directory_( directory ) , sessionId_( id ) , qNeedToMkdir( 1 ) , db_( db ) ,
    dbSessionId_( dbSessionId )
{
  printf("Record_Archive_Manager::Record_Archive_Manager: creating!\n");
}

Record_Archive_Manager::~Record_Archive_Manager( )
{
    printf("Record_Archive_Manager::~Record_Archive_Manager:  destructing!\n");
}

void
Record_Archive_Manager::record( RTSP_Record_Request* rRequest )
{
  printf("Record_Archive_Manager::record()\n");

    RTSP_Request_Packet* request = rRequest->requestPacket( );
    RTSP_Response resp( 200 , "OK" , atoi( request->sequence( ) ) );

    char sessionId[ 256 ];
    sprintf( sessionId , "%d" , ( int )sessionId_ );
    resp.setHeader( "Session" , sessionId );
    resp.send( request->engine( ) );
    
    enableRecording();
}

void
Record_Archive_Manager::handlePacket(RTP_Packet::RTP_Header *packetHeader,
				char *packet, int packetSize,
				ACE_INET_Addr &packetSource,
				ACE_SOCK_Dgram *sourceSocket)
{
    Stream_Archive *streamArchive;
    int qFirstPacket = 0;

  printf("Record_Archive_Manager::handlePacket() RTP\n");

    streamArchive = streamHash_.lookup(packetHeader->get_ssrc());
    if( streamArchive == ( Stream_Archive* )NULL )
    {	
	streamArchive = createStreamArchive( packetHeader->get_ssrc( ) );
	if( streamArchive )
	{
	    streamHash_.add( packetHeader->get_ssrc( ) , streamArchive );
	    qFirstPacket = 1;
	}
	else
	{
	    // exception
	    assert( 0 );
	}
    }

    assert( streamArchive );
    streamArchive->handlePacket( packetHeader , packet , packetSize ,
			       packetSource , qFirstPacket  );
}
    
void
Record_Archive_Manager::handlePacket(RTCP_Packet::RTCP_Header *packetHeader,
				char *packet, int packetSize,
				ACE_INET_Addr &packetSource,
				ACE_SOCK_Dgram *sourceSocket)
{
    Stream_Archive* streamArchive;
    int qFirstPacket = 0;

  printf("Record_Archive_Manager::handlePacket() RTCP\n");

    streamArchive = streamHash_.lookup( packetHeader->get_ssrc( ) );
    if( streamArchive == ( Stream_Archive* )NULL )
    {
	streamArchive = createStreamArchive( packetHeader->get_ssrc( ) );
	streamHash_.add( packetHeader->get_ssrc() , streamArchive );
	qFirstPacket = 1;
    }
    assert( streamArchive );
    streamArchive->handlePacket( packetHeader , packet , packetSize ,  packetSource, qFirstPacket );
}

void
Record_Archive_Manager::handleSubpacket(RTCP_Packet::RTCP_Header *packetHeader ,
				   char *packet , int packetSize ,
				   ACE_INET_Addr &packetSource ,
				   ACE_SOCK_Dgram *sourceSocket )
{
    Stream_Archive* streamArchive;

    printf("Record_Archive_Manager::handleSubpacket()\n");

    streamArchive = streamHash_.lookup(packetHeader->get_ssrc());
    if (streamArchive == ( Stream_Archive* ) NULL)
    {
	streamArchive = createStreamArchive( packetHeader->get_ssrc( ) );
	streamHash_.add( packetHeader->get_ssrc( ) , streamArchive );
    }

    assert( streamArchive );
    streamArchive->handleSubpacket( packetHeader , packet , packetSize ,  packetSource );
}

void
Record_Archive_Manager::enableRecording( )
{
    if( qNeedToMkdir && ACE_OS::mkdir( directory_.c_str( ) ) != 0 )
    {
	printf( "Record_Archive_Manager::enableRecording:  mkdir %s failed:  %s\n" ,
		directory_.c_str( ) , strerror( ACE_OS::last_error( ) ) );
	assert( 0 );
    }
    else
    {
	qNeedToMkdir = 0;
	recordFlag_ = 1;
	printf( "Record_Archive_Manager::enableRecording:  Enabling recording\n" );
    }
}

void
Record_Archive_Manager::disableRecording( void )
{
    recordFlag_ = 0;
    printf( "Record_Archive_Manager::disableRecording:  Disabling recording\n" );
}

void
Record_Archive_Manager::endOfStream( Stream_Archive* streamArch )
{
    printf( "Record_Archive_Manager::endOfStream: got finished for ssrc %x\n" , streamArch->ssrc( ) );

    streamHash_.remove( streamArch->ssrc( ) );
    delete streamArch;
}

void
Record_Archive_Manager::terminate( void )
{
    streamHash_.callTerminate( );
}


int
Record_Archive_Manager::handle_timeout (const ACE_Time_Value& tv , const void* arg )
{
    ACE_Time_Value delta = tv - lastTick_;
    
    streamHash_.callTick( tv , lastTick_ , delta );
    lastTick_ = tv;
    
    return 0;
}

Stream_Archive*
Record_Archive_Manager::createStreamArchive( long ssrc )
{
    Stream_Archive* streamArchive = NULL;
    char streamId[ 128 ];
    
    sprintf( streamId , "%x" , ssrc );

    //
    // THIS IS THE PLACE WHERE streamId becomes BOUND TO ssrc
    //
    streamArchive = new Stream_Archive( this , ssrc , db_ ,  ( long )dbSessionId_ , streamId );
    
    //    printf( "Record_Archive_Manager::createStreamArchive: dbSession = %s, ssrc = %x, streamArch = %u\n" , dbSessionId_ , ssrc , streamArchive );
	
    return streamArchive;
}
  
