// Record_Listener.C
//
// Record_Listener 
//
// Revision History
//     13 Jan 98 - lisa - adapted from prototype recording code
//

#include "Record_Listener.h"
#include "RTP_Listener.h"
#include "RTCP_Listener.h"
#include "Record_Session_Manager.h"
#include "Record_Network_Transport.h"
#include "ace/Reactor.h"
#include "RTSP_Record_Request.h"
#include "Record_Archive_Manager.h"
#include "Transport_Manager.h"


Record_Listener::Record_Listener( Record_Session_Manager* recMgr ,
				  Record_Network_Transport* netTrans ,
				  Record_Archive_Manager* archiveMgr ,
				  ACE_Reactor* reactor) :
    recordMgr_( recMgr ) ,  netTrans_( netTrans ) , reactor_( reactor ) , archiveMgr_( archiveMgr ) 
{
    assert( recMgr && netTrans && archiveMgr && reactor );

    rtpListener_ = new RTP_Listener( netTrans->rtpSocket( ) ,archiveMgr_ , reactor_ );
    rtcpListener_ = new RTCP_Listener( netTrans->rtcpSocket( ) , archiveMgr_  , reactor_ );

    assert( rtpListener_ && rtcpListener_ );
}

Record_Listener::~Record_Listener( void )
{
    if( rtpListener_ )
	delete rtpListener_;
    if( rtcpListener_ )
	delete rtcpListener_;
}

void
Record_Listener::record( RTSP_Record_Request* request )
{
    assert( request && rtpListener_  && rtcpListener_  );
    printf( "Record_Listener::record %u\n" , this );
}

void
Record_Listener::teardown( Transport_Manager* transMan )
{
    if( rtpListener_ )
    {
	delete rtpListener_;
	rtpListener_ = NULL;
    }

    if( rtcpListener_ )
    {
	delete rtcpListener_;
	rtcpListener_ = NULL;
    }

    if( transMan )
	transMan->closeTransport( netTrans_ );
}
