#include "Record_Network_Transport.h"
#include "RTP_Packet.h"

/////////////////////////////////////////////////////////////////////////////
//
// Record_Network_Transport.C
//
// Revision History
//      26 jan 98- lisa - adapted from Network_Transport
//

#include <sys/errno.h>
#include <stream.h>
#include <ace/Reactor.h>

Record_Network_Transport::Record_Network_Transport( Network_Endpoint ePoint ) :
    endpoint_( ePoint ) , rtp_address_( ePoint.rtpAddress( ) ) , qError_( 0 ) ,
    rtcp_address_( ePoint.rtcpAddress( ) ) , isMulticast_( ePoint.isMulticast( ) )
{
    cout << form( "Record_Network_Transport::Record_Network_Transport: %x for %s/%d\n" ,
		  this , rtp_address_.get_host_addr( ) , rtp_address_.get_port_number( ) );
    cout << "Record_Network_Transport::Record_Network_Transport: endpoint is " <<
	ePoint << endl;


    if( isMulticast_ )
    {
	int rc;
	
	rc = rtp_mc_sock_.subscribe( rtp_address_  );
	if( rc < 0 )
	{
	    perror("Record_Network_Transport::Record_Network_Transport: subscribe rtp error");
	    qError_ = 1;
	}
	    
	rc = rtcp_mc_sock_.subscribe(rtcp_address_ );
	if( rc <  0)
	{
	    perror("Record_Network_Transport::Record_Network_Transport: subscribe rtcp error");
	    qError_ = 1;
	}
	    
	//ePoint.ttl_ = 16;
	if( ePoint.ttl_ > 1 )
	{
	    if( rtp_mc_sock_.set_option(IP_MULTICAST_TTL, ePoint.ttl_ ) < 0 )
	    {
		perror("Record_Network_Transport::Record_Network_Transport: rtp set ttl error");
		qError_ = 1;
	    }
	    if( rtcp_mc_sock_.set_option(IP_MULTICAST_TTL, ePoint.ttl_ ) < 0 )
	    {
		perror("Record_Network_Transport::Record_Network_Transport: rtcp set ttl error");
		qError_ = 1;
	    }
	    if( rtp_mc_sock_.set_option(IP_MULTICAST_LOOP, 0) < 0 )
	    {
		perror("Record_Network_Transport::Record_Network_Transport: rtp set loop error");
		qError_ = 1;
	    }
	    if( rtcp_mc_sock_.set_option(IP_MULTICAST_LOOP, 0) < 0 )
	    {
		perror("Record_Network_Transport::Record_Network_Transport: rtcp set loop error");
		qError_ = 1;
	    }
	}
    }
    else
    {
	if( rtp_sock_.open(ACE_INET_Addr(ePoint.port_)) < 0 )
	{
	    perror("Record_Network_Transport::Record_Network_Transport: rtp open error");
	    qError_ = 1;
	}

	if( rtcp_sock_.open(ACE_INET_Addr(ePoint.port_ + 1 ) ) < 0 )
	{
	    perror("Record_Network_Transport::Record_Network_Transport: rtcp open error");
	    qError_ = 1;
	}
    }
}

Record_Network_Transport::~Record_Network_Transport( )
{
    printf( "Record_Network_Transport::~Record_Network_Transport: %x\n" , this );
    if( isMulticast_ )
    {
	printf( "Record_Network_Transport::~Record_Network_Transport: unsubscribing to mc session\n" );
	if( rtp_mc_sock_.unsubscribe( ) < 0 )
	{
	    printf( "Record_Network_Transport::~Record_Network_Transport: unsubscribe error\n" );
	}
	else
	{
	    int handle = rtp_mc_sock_.get_handle( );
	    int rc = rtp_mc_sock_.close( );
	    printf( "Record_Network_Transport::~Record_Network_Transport:  closed rtp socket %x, handle %d, close rc %d\n" , &rtp_mc_sock_ , handle , rc );
	}
	
	if( rtcp_mc_sock_.unsubscribe( ) < 0 )
	{
	    printf( "Record_Network_Transport::~Record_Network_Transport: unsubscribe error\n" );
	}
	else
	{
	    int handle = rtcp_mc_sock_.get_handle( );
	    int rc = rtcp_mc_sock_.close( );
	    printf( "Record_Network_Transport::~Record_Network_Transport:  closed rtcp socket %x, handle %d, close rc %d\n" , &rtcp_mc_sock_ , handle , rc );
	}
    }
    else
    {
	printf( "Record_Network_Transport::~Record_Network_Transport: closing unicast sockets\n" );
	rtp_sock_.close( );
	rtcp_sock_.close( );
    }
}

ACE_SOCK_Dgram*
Record_Network_Transport::rtpSocket( void )
{
    return isMulticast_ ? &rtp_mc_sock_ : &rtp_sock_;
}

ACE_SOCK_Dgram*
Record_Network_Transport::rtcpSocket( void )
{
    return isMulticast_ ? &rtcp_mc_sock_ : &rtcp_sock_;
}
