#include "config.h"
#include "Stream_Archive_Hash.h"
#include "Stream_Archive.h"

Stream_Archive_Hash::Stream_Archive_Hash(int size) : size_( size )
{
    elements_ = new Stream_Archive_Hash_Bucket [size_];

    for (int i = 0; i < size_; i++)
    {
	elements_[i].next_ = 0;
    }
}

Stream_Archive_Hash::~Stream_Archive_Hash()
{
    delete elements_;
}

void Stream_Archive_Hash::add(long ssrc, Stream_Archive* streamArchive)
{
    Stream_Archive_Hash_Bucket *b, *nb;

    b = getBucket(ssrc);
    nb = new Stream_Archive_Hash_Bucket(ssrc, streamArchive, b->next_);
    b->next_ = nb;
}

void Stream_Archive_Hash::remove(long ssrc)
{
    Stream_Archive_Hash_Bucket *b, *lb;

    lb = getBucket(ssrc);
    b = lb->next_;
    
    while (b != NULL && b->ssrc_ != ssrc)
    {
	b = b->next_;
	lb = b;
    }

    if (b)
    {
	if (b->ssrc_ != ssrc)
	{
	    printf("aiee\n");
	    exit(1);
	}
	else
	{
	    lb->next_ = b->next_;
	}
    }
    else
    {
	printf("Stream_Archive_Hash::remove: didn't find ssrc %x\n", ssrc);
    }
}

Stream_Archive*
Stream_Archive_Hash::lookup(long ssrc)
{
    Stream_Archive_Hash_Bucket *b;

    b = getBucket(ssrc);

    b = b->next_;
    while (b)
    {
	if (b->ssrc_ == ssrc)
	    return b->streamArchive_;
	b = b->next_;
    }
    return 0;
}

Stream_Archive_Hash_Bucket *
Stream_Archive_Hash::getBucket(long ssrc)
{
    unsigned int hash;

    hash = (unsigned int) ssrc % size_;
    return &(elements_[hash]);
}

void Stream_Archive_Hash::callTick(ACE_Time_Value tv, ACE_Time_Value last_tv, ACE_Time_Value delta)
{
    Stream_Archive_Hash_Bucket *b;
    int i;

    for (i = 0; i < size_; i++)
    {
	for (b = elements_[i].next_; b != NULL; b = b->next_)
	{
	    if (b->streamArchive_ != NULL)
	    {
	        b->streamArchive_->tick(tv, last_tv, delta);
	    }
	}
    }
}

void Stream_Archive_Hash::callTerminate()
{
    Stream_Archive_Hash_Bucket *b;
    int i;

    for (i = 0; i < size_; i++)
    {
	for (b = elements_[i].next_; b != NULL; b = b->next_)
	{
	    if (b->streamArchive_ != NULL)
	    {
	        b->streamArchive_->terminate();
	    }
	}
    }
}

Stream_Archive_Hash_Bucket::Stream_Archive_Hash_Bucket( long ssrc , Stream_Archive* streamArchive ,
				   Stream_Archive_Hash_Bucket* next ) :
				   ssrc_( ssrc ) , streamArchive_( streamArchive ) , next_( next )
{
}

Stream_Archive_Hash_Bucket::Stream_Archive_Hash_Bucket( ) :
				   ssrc_( 0L ) ,
				   streamArchive_( 0 ) ,
				   next_( 0 )
{
}

