#include "Voyager_Server.h"
//#include "Voyager_DB.h"
#include "RTSP_Acceptor.h"
#include "Session_Manager.h"
#include "Dispatcher.h"
#include "Playback_Manager.h"
#include "Record_Manager.h"
#include "Transport_Manager.h"
#include "Corba_Manager.h"

#include "VOY_Database.h"

#include <ace/Reactor.h>
#include <ace/Get_Opt.h>

Voyager_Server::Voyager_Server() :
    listenPort_(5400),
    refFile_("../db/VDI.ref"),
    sessionManager_(0),
    acceptor_(0),
    dispatcher_(0),
    transportManager_(0),
    playbackManager_(0),
    recordManager_(0),
    corbaManager_(0),
    payFreqFileName_( "" )
{
}

//
// Need to init corba first, in case it wants to peel some
// arguments off.
//
void Voyager_Server::init(int &argc, char **argv)
{
    initCorba(argc, argv);

    // Process Args moved into initCorba (this is gross)

    initServerObjects();

    initAcceptor();
}

void Voyager_Server::initCorba(int &argc, char **argv)
{
    corbaManager_ = new Corba_Manager();
    corbaManager_->open(argc, argv);

    processArgs(argc, argv);
    
    CORBA_Object_ptr obj = corbaManager_->objectFromIORFile(refFile_);

    databaseRef_ = VOY_Database::_narrow(obj);
    assert(!CORBA_is_nil(databaseRef_));
}

void Voyager_Server::initServerObjects()
{
  sessionManager_ = new Session_Manager(databaseRef_);

  transportManager_ = new Transport_Manager();

  playbackManager_ = new Playback_Manager(transportManager_, databaseRef_ ,
					  payFreqFileName_ );

  recordManager_ = new Record_Manager(transportManager_, databaseRef_);

  dispatcher_ = new Dispatcher(sessionManager_, playbackManager_,
			       recordManager_);
}

void Voyager_Server::initAcceptor()
{
    ACE_INET_Addr addr(listenPort_), myaddr;

    acceptor_ = new RTSP_Acceptor(dispatcher_);

    acceptor_->open(addr);

    acceptor_->acceptor().get_local_addr(myaddr);

    printf("Acceptor listening on %d\n", myaddr.get_port_number());
}


void Voyager_Server::processArgs(int &argc, char **argv)
{
    ACE_Get_Opt get_opt(argc, argv, "p:c:f:");
    int c;

    while ((c = get_opt()) != -1)
    {
	switch (c)
	{
	case 'p':
	    listenPort_ = atoi(get_opt.optarg);
	    break;

	case 'f':
	  refFile_ = ACE_OS::strdup(get_opt.optarg);
	  break;

	case 'c':
	  payFreqFileName_ = get_opt.optarg;
	  break;
	  
	default:
	    printf("usage: %s -p rtsp-port  -f <filespec for the reffile>  [ -c <filespec for the payload-frequency table> ]\n", argv[0]);
	    exit(1);
	}
    }
}

void Voyager_Server::run()
{
    done_ = 0;
    while (!done_)
    {
	ACE_Reactor::instance()->handle_events();
    }
}
