#include "Voyager_Session.h"
#include "RTSP_Request_Packet.h"
#include "RTSP_Describe_Request.h"
#include "RTSP_Response.h"
#include "RTSP_Engine.h"
//#include "Voyager_DB.h"

//#include <zdb.h>

Voyager_Session::Voyager_Session(Voyager_Session_ID id) :
    id_(id)
{
}

Voyager_Session *Voyager_Session::loadFromDB(Voyager_Session_ID id,
					     VOY_Database_var db)
{
  char *sessionName_;
  char *description_;
  StringSeq_var streams = new StringSeq;
  Voyager_Session *session = new Voyager_Session(id);
  
  sessionName_ = db->getName(id);
  description_ = db->getDescription(id);

  session->setName(sessionName_);
  session->setDescription(description_);

  //
  // Now load in the names of the streams, create stream objects
  // for them, and have the stream objects finish loading the stream info.
  //

  streams = db->getStreamNames(id);

  for(int i = 0; i < streams->length(); i++)
    {
      Voyager_Stream *stream = new Voyager_Stream(id, streams[i]);
      session->addStream(stream);
    }

  Stream_Map::iterator iter;
  for(iter = session->streamMap_.begin(); iter != session->streamMap_.end(); iter++)
    {
      Voyager_Stream *s = (*iter).second;
      s->loadFromDB(db);
    }

    return session;
}

// Voyager_Session *Voyager_Session::loadFromDB(Voyager_Session_ID id,
// 					     Voyager_DB *db)
// {
//     //
//     // Get the session info for our session
//     //

//     ZSQL *sql = db->sql();

//     if (sql->Open("select name, description from Session where id = %d",
// 		  (int) id)
// 	== 0)
//     {
// 	printf("query failed\n");
// 	return NULL;
//     }

//     int nRows = sql->GetNumRows();
//     if (nRows <= 0)
//     {
// 	printf("Session not found\n");
// 	return NULL;
//     }
//     else if (nRows != 1)
//     {
// 	printf("Nonunique session id?\n");
//     }

//     //
//     // Get the size of the name/description fields
//     //
//     ZField fieldInfo;

//     sql->GetNextRecord();

//     sql->GetFieldInfo(0, &fieldInfo);
//     char *name = new char[fieldInfo.Length + 1];
//     sql->GetSQLData(0, name, fieldInfo.Length);

//     sql->GetFieldInfo(1, &fieldInfo);
//     char *descr = new char[fieldInfo.Length + 1];
//     sql->GetSQLData(1, descr, fieldInfo.Length);

//     printf("name='%s' desc='%s'\n", name, descr);

//     Voyager_Session *s = new Voyager_Session(id);

//     s->setName(name);
//     s->setDescription(descr);

//     sql->Close();

//     //
//     // Now load in the names of the streams, create stream objects
//     // for them, and have the stream objects finish loading the stream info.
//     //

//     sql->Open("select distinct ssrc from SessionMetadata where id = %d",
// 	      (int) id);
//     char ssrc[100];
//     while (sql->GetNextRecord())
//     {
// 	sql->GetSQLData(0, ssrc, sizeof(ssrc));
// 	Voyager_Stream *stream = new Voyager_Stream(id, ACE_OS::strdup(ssrc));
// 	s->addStream(stream);
//     }

//     //
//     // Must load streams after we're done with this query, since
//     // we just get one active query at a time (at least unless we
//     // create a new ZSQL for each one).
//     //

//     Stream_Map::iterator iter;

//     for (iter = s->streamMap_.begin(); iter != s->streamMap_.end(); iter++)
//     {
// 	Voyager_Stream *stream = (*iter).second;

// 	stream->loadFromDB(db);
//     }
//     return s;
// }

void Voyager_Session::addStream(Voyager_Stream *stream)
{
    printf("Add stream %d %s\n", (int) stream->sessionId(), stream->ssrc());

    streamMap_[stream->ssrc()] = stream;
}

void Voyager_Session::handleDescribeSession(RTSP_Describe_Request *drequest)
{
    RTSP_Request_Packet *request = drequest->requestPacket();
    
    printf("Voyager session gets describeSession %s %s %d\n",
	   request->uri(), request->path(), request->sequence());

    RTSP_Response r(200, "OK", atoi(request->sequence()));

//    r.setHeader("Content-Type", "application/x-rtsp-mh");
    r.setHeader("Content-Type", "x-bob/session-description");
    r.bodyInit(1000);

    //
    // Send the # of streams and the ssrc ids
    //

    r.bodyAppendf("%d\n", streamMap_.size());

    Stream_Map::iterator iter;

    for (iter = streamMap_.begin(); iter != streamMap_.end(); iter++)
    {
	r.bodyAppendf("%s\n", (*iter).first.c_str());
    }

    //
    // send the name, then the description ends the body
    //

    r.bodyAppendf("%s\n", name());
    r.bodyAppendf("%s\n", description());
    r.bodyDone();

    r.send(request->engine());
}

void Voyager_Session::handleDescribeStream(RTSP_Describe_Request *drequest,
					   char *ssrc)
{
    RTSP_Request_Packet *request = drequest->requestPacket();

    printf("Voyager session gets describeStream '%s'\n", ssrc);

    Voyager_Stream *stream;

    stream = streamMap_[ssrc];
    if (stream != 0)
    {
	stream->handleDescribeStream(drequest);
    }
    else
    {
	RTSP_Response r(500, "Stream not found", request->sequence());
	r.send(request->engine());
    }
}
