/*
 * Generate a random 32-bit quantity.
 */
#include <sys/types.h>   /* u_long */
#include <sys/time.h>    /* gettimeofday() */
#include <unistd.h>      /* get..() */
#include <stdio.h>       /* printf() */
#include <time.h>        /* clock() */
#include <sys/utsname.h> /* uname() */
extern "C" {
#include "global.h"      /* from RFC 1321 */
#include "md5.h"         /* from RFC 1321 */
}

#define MD_CTX MD5_CTX
#define MDInit MD5Init
#define MDUpdate MD5Update
#define MDFinal MD5Final

static u_long md_32(char *string, int length)
{
    MD_CTX context;
    union {
        char   c[16];
        u_long x[4];
    } digest;
    u_long r;
    int i;

    MDInit (&context);
    MDUpdate (&context, (unsigned char *) string, length);
    MDFinal ((unsigned char *)&digest, &context);
    r = 0;
    for (i = 0; i < 3; i++) {
        r ^= digest.x[i];
    }
    return r;
}                               /* md_32 */


/*
 * Return random unsigned 32-bit quantity. Use 'type' argument if you
 * need to generate several different values in close succession.
 */
UINT4 random32(int type)
{
    struct {
        int     type;
        struct  timeval tv;
        clock_t cpu;
        pid_t   pid;
        u_long  hid;
        uid_t   uid;
        gid_t   gid;
        struct  utsname name;
    } s;

    gettimeofday(&s.tv, 0);
    uname(&s.name);
    s.type = type;
    s.cpu  = clock();
    s.pid  = getpid();
    s.hid  = gethostid();
    s.uid  = getuid();
    s.gid  = getgid();

    return md_32((char *)&s, sizeof(s));
}                               /* random32 */

int getRandomPort()
{
    static int foob = 0;
    UINT4 r;
    int p;

    do {
	r = random32(foob++);
	p = r & 0xfffe;
    } while (p <= 10000);

    return p;
}

#ifdef TEST_RANDOM

static char hits[100000];
main(int argc, char **argv)
{
    int i;

    for (i = 0; ; i++)
    {
	int p = getRandomPort();
	if (hits[p] > 0)
	{
	    printf("Hit on %d at %d\n", p, i);
	}
	hits[p]++;
//	printf("%d\n", p);
    }
}
#endif
